# -*- coding: utf-8 -*-
from django.test import TestCase
from django.contrib.auth.models import User
from django.core.urlresolvers import reverse
from django.test.client import RequestFactory
from django.contrib.admin.sites import AdminSite
from django.contrib import messages
from django.contrib.messages.storage.fallback import FallbackStorage

from mock import Mock, patch

from unisender.models import (
    Tag, Field, SubscribeList, Subscriber, EmailMessage, Campaign)

from unisender.admin import (
    FieldAdmin, SubscribeListAdmin, SubscriberAdmin, EmailMessageAdmin,
    CampaignAdmin, AttachmentInline, AttachmentInlineReadOnly
)

from unisender.unisender_urls import (
    EMAIL_MESSAGES_LIST, EMAIL_MESSAGES_DETAIL, TAG_LIST, FIELD_LIST,
    CAMPAIGN_LIST, CAMPAIGN_DETAIL, SUBSCRIBELIST_LIST, SUBSCRIBELIST_DETAIL
)

from .mock_api import mock_messages


def get_csrf_token(response):
    csrf = "name='csrfmiddlewaretoken' value='"
    start = response.content.find(csrf) + len(csrf)
    end = response.content.find("'", start)

    return response.content[start:end]


class UnisenderAdminTestCase(TestCase):

    def setUp(self):
        username = 'test_user'
        pwd = 'secret'

        self.user = User.objects.create_user(username, '', pwd)
        self.user.is_staff = True
        self.user.is_superuser = True
        self.user.save()

        self.assertTrue(self.client.login(username=username, password=pwd),
                        "Logging in user %s, pwd %s failed." % (username, pwd))

    def test_open_unisender_app(self):
        """Открываем страницу приложения unisneder в админке"""
        response = self.client.get(
            reverse('admin:app_list', kwargs={'app_label': 'unisender'}))
        self.assertEqual(response.status_code, 200)


class TagAdminTestCase(TestCase):

    def setUp(self):
        username = 'test_user'
        pwd = 'secret'

        self.user = User.objects.create_user(username, '', pwd)
        self.user.is_staff = True
        self.user.is_superuser = True
        self.user.save()

        self.assertTrue(self.client.login(username=username, password=pwd),
                        "Logging in user %s, pwd %s failed." % (username, pwd))

    def test_open_unisender_page(self):
        """Открываем страницу Tags в админке"""
        response = self.client.get(reverse('admin:unisender_tag_changelist'))
        self.assertEqual(response.status_code, 200)
        self.assertIn('unisender_site', response.context)
        self.assertEqual(response.context['unisender_site'], TAG_LIST)

    def test_add_tag(self):
        """Добавляем тэг"""
        start_tag_count = Tag.objects.count()
        url = reverse('admin:unisender_tag_add')
        response = self.client.get(url)
        self.assertEqual(response.status_code, 200)
        csrf = get_csrf_token(response)
        post_data = {'name': u'test_tag',
                     'csrfmiddlewaretoken': csrf, }
        response = self.client.post(url, post_data)
        self.assertRedirects(
            response, reverse('admin:unisender_tag_changelist'))
        self.assertEqual(Tag.objects.count(), start_tag_count + 1)

    def test_update_tag(self):
        """Редактируем тэг"""
        tag = Tag.objects.create(name='test')
        url = reverse('admin:unisender_tag_change', args=(tag.pk,))
        response = self.client.get(url)
        self.assertEqual(response.status_code, 200)
        csrf = get_csrf_token(response)
        post_data = {'name': u'test_tag',
                     'csrfmiddlewaretoken': csrf,
                     }
        response = self.client.post(url, post_data)
        self.assertRedirects(
            response, reverse('admin:unisender_tag_changelist'))
        self.assertTrue(Tag.objects.filter(name='test_tag').exists())

    def test_delete_tag(self):
        """Удаляем тэг"""
        start_unisender_count = Tag.objects.count()
        tag = Tag.objects.create(name='test')
        url = reverse('admin:unisender_tag_delete', args=(tag.pk,))
        response = self.client.get(url)
        self.assertEqual(response.status_code, 200)
        csrf = get_csrf_token(response)
        post_data = {'csrfmiddlewaretoken': csrf}
        response = self.client.post(url, post_data)
        self.assertRedirects(
            response, reverse('admin:unisender_tag_changelist'))
        self.assertEqual(Tag.objects.count(), start_unisender_count)


class FieldAdminTestCase(TestCase):

    def setUp(self):
        username = 'test_user'
        pwd = 'secret'

        self.user = User.objects.create_user(username, '', pwd)
        self.user.is_staff = True
        self.user.is_superuser = True
        self.user.save()
        self.assertTrue(self.client.login(username=username, password=pwd),
                        "Logging in user %s, pwd %s failed." % (username, pwd))
        site = AdminSite()
        self.admin = FieldAdmin(Field, site)
        self.request = RequestFactory().get(reverse('admin:index'))

    def test_open_unisender_page(self):
        """Открываем страницу Tags в админке"""
        response = self.client.get(reverse('admin:unisender_field_changelist'))
        self.assertEqual(response.status_code, 200)
        self.assertIn('unisender_site', response.context)
        self.assertEqual(response.context['unisender_site'], FIELD_LIST)

    def test_add_field(self):
        """Добавляем поле"""
        start_field_count = Field.objects.count()
        url = reverse('admin:unisender_field_add')
        response = self.client.get(url)
        self.assertEqual(response.status_code, 200)
        csrf = get_csrf_token(response)
        post_data = {'name': u'test_field',
                     'csrfmiddlewaretoken': csrf,
                     'sort': 1,
                     'field_type': 'string'}
        response = self.client.post(url, post_data)
        self.assertRedirects(
            response, reverse('admin:unisender_field_changelist'))
        self.assertEqual(Field.objects.count(), start_field_count + 1)

    def test_update_field(self):
        """Редактируем поле"""
        field = Field.objects.create(name='test')
        url = reverse('admin:unisender_field_change', args=(field.pk,))
        response = self.client.get(url)
        self.assertEqual(response.status_code, 200)
        csrf = get_csrf_token(response)
        post_data = {'name': u'test_field',
                     'csrfmiddlewaretoken': csrf,
                     'sort': 1,
                     'field_type': 'string'
                     }
        response = self.client.post(url, post_data)
        self.assertRedirects(
            response, reverse('admin:unisender_field_changelist'))
        self.assertTrue(Field.objects.filter(name='test_field').exists())

    def test_delete_field(self):
        """Удаляем поле"""
        start_unisender_count = Field.objects.count()
        field = Field.objects.create(name='test')
        url = reverse('admin:unisender_field_delete', args=(field.pk,))
        response = self.client.get(url)
        self.assertEqual(response.status_code, 200)
        csrf = get_csrf_token(response)
        post_data = {'csrfmiddlewaretoken': csrf}
        response = self.client.post(url, post_data)
        self.assertRedirects(
            response, reverse('admin:unisender_field_changelist'))
        self.assertEqual(Field.objects.count(), start_unisender_count)

    def test_admin_get_actions(self):
        actions = self.admin.get_actions(self.request)
        self.assertEqual(len(actions.keys()), 1)
        self.assertIn('delete_selected_fields', actions)

    def test_delete_selected_fields(self):
        start_unisender_count = Field.objects.count()
        Field.delete_field = Mock(return_value=None)
        for x in xrange(3):
            Field.objects.create(name='test')
        queryset = Field.objects.all()
        self.admin.delete_selected_fields(self.request, queryset)
        self.assertTrue(Field.delete_field.called)
        self.assertEqual(Field.delete_field.call_count, 3)
        self.assertEqual(Field.objects.count(), start_unisender_count)

    def test_save_model(self):
        start_unisender_count = Field.objects.count()
        field = Field.objects.create(name='test', unisender_id=1)
        field.update_field = Mock(return_value=None)
        self.admin.save_model(self.request, field, None, None)
        self.assertEqual(Field.objects.count(), start_unisender_count + 1)
        self.assertTrue(field.update_field.called)
        self.assertEqual(field.update_field.call_count, 1)
        field_2 = Field.objects.create(name='test_2')
        field_2.create_field = Mock(return_value=None)
        self.admin.save_model(self.request, field_2, None, None)
        self.assertEqual(Field.objects.count(), start_unisender_count + 2)
        self.assertTrue(field_2.create_field.called)
        self.assertEqual(field_2.create_field.call_count, 1)

    def test_delete_model(self):
        start_unisender_count = Field.objects.count()
        field = Field.objects.create(name='test')
        field.delete_field = Mock(return_value=None)
        self.admin.delete_model(self.request, field)
        self.assertEqual(Field.objects.count(), start_unisender_count)
        self.assertTrue(field.delete_field.called)
        self.assertEqual(field.delete_field.call_count, 1)


class SubscribeListAdminTestCase(TestCase):

    def setUp(self):
        username = 'test_user'
        pwd = 'secret'

        self.user = User.objects.create_user(username, '', pwd)
        self.user.is_staff = True
        self.user.is_superuser = True
        self.user.save()
        self.assertTrue(self.client.login(username=username, password=pwd),
                        "Logging in user %s, pwd %s failed." % (username, pwd))
        site = AdminSite()
        self.admin = SubscribeListAdmin(SubscribeList, site)
        self.request = RequestFactory().get(reverse('admin:index'))

    def test_open_unisender_page(self):
        """Открываем страницу subscribeList в админке"""
        response = self.client.get(
            reverse('admin:unisender_subscribelist_changelist'))
        self.assertEqual(response.status_code, 200)
        self.assertIn('unisender_site', response.context)
        self.assertEqual(response.context['unisender_site'], SUBSCRIBELIST_LIST)

    def test_add_subscribe_list(self):
        """Добавляем список рассылки"""
        start_subscribe_list_count = SubscribeList.objects.count()
        url = reverse('admin:unisender_subscribelist_add')
        response = self.client.get(url)
        self.assertEqual(response.status_code, 200)
        csrf = get_csrf_token(response)
        post_data = {'title': u'test_subscribe_list',
                     'csrfmiddlewaretoken': csrf,
                     }
        response = self.client.post(url, post_data)
        self.assertRedirects(
            response, reverse('admin:unisender_subscribelist_changelist'))
        self.assertEqual(
            SubscribeList.objects.count(), start_subscribe_list_count + 1)

    def test_update_subscribe_list(self):
        """Редактируем список рассылки"""
        subscribe_list = SubscribeList.objects.create(
            title='test', unisender_id=1)
        url = reverse(
            'admin:unisender_subscribelist_change', args=(subscribe_list.pk,))
        response = self.client.get(url)
        self.assertEqual(response.status_code, 200)
        self.assertIn('unisender_site', response.context)
        self.assertEqual(
            response.context['unisender_site'],
            SUBSCRIBELIST_DETAIL + str(subscribe_list.unisender_id))
        csrf = get_csrf_token(response)
        post_data = {'title': u'test_subscribe_list',
                     'csrfmiddlewaretoken': csrf,
                     'optinemail-TOTAL_FORMS': 1,
                     'optinemail-INITIAL_FORMS': 0,
                     'optinemail-MAX_NUM_FORMS': 1,
                     'optinemail-0-sender_name': 'test_name',
                     'optinemail-0-sender_email': 'test@example.com',
                     'optinemail-0-subject': 'test',
                     'optinemail-0-body': 'test',
                     'optinemail-0-list_id': subscribe_list.pk,
                     }
        response = self.client.post(url, post_data)
        # print '\n%s\n' % response
        self.assertRedirects(
            response, reverse('admin:unisender_subscribelist_changelist'))
        self.assertTrue(SubscribeList.objects.filter(
            title='test_subscribe_list').exists())

    def test_delete_subscribe_list(self):
        """Удаляем список рассылки"""
        start_unisender_count = SubscribeList.objects.count()
        subscribe_list = SubscribeList.objects.create(title='test')
        url = reverse(
            'admin:unisender_subscribelist_delete', args=(subscribe_list.pk,))
        response = self.client.get(url)
        self.assertEqual(response.status_code, 200)
        csrf = get_csrf_token(response)
        post_data = {'csrfmiddlewaretoken': csrf}
        response = self.client.post(url, post_data)
        self.assertRedirects(
            response, reverse('admin:unisender_subscribelist_changelist'))
        self.assertEqual(SubscribeList.objects.count(), start_unisender_count)

    def test_admin_get_actions(self):
        actions = self.admin.get_actions(self.request)
        self.assertEqual(len(actions.keys()), 1)
        self.assertIn('delete_selected_subscribe_list', actions)

    def test_delete_selected_lists(self):
        start_unisender_count = SubscribeList.objects.count()
        SubscribeList.delete_list = Mock(return_value=None)
        for x in xrange(3):
            SubscribeList.objects.create(title='test_%s' % x)
        queryset = SubscribeList.objects.all()
        self.admin.delete_selected_subscribe_list(self.request, queryset)
        self.assertTrue(SubscribeList.delete_list.called)
        self.assertEqual(SubscribeList.delete_list.call_count, 3)
        self.assertEqual(SubscribeList.objects.count(), start_unisender_count)

    def test_save_model(self):
        start_unisender_count = SubscribeList.objects.count()
        subscribe_list = SubscribeList.objects.create(
            title='test', unisender_id=1)
        subscribe_list.update_list = Mock(return_value=None)
        self.admin.save_model(self.request, subscribe_list, None, None)
        self.assertEqual(
            SubscribeList.objects.count(), start_unisender_count + 1)
        self.assertTrue(subscribe_list.update_list.called)
        self.assertEqual(subscribe_list.update_list.call_count, 1)

        subscribe_list_2 = SubscribeList.objects.create(title='test_2')
        subscribe_list_2.create_list = Mock(return_value=None)
        self.admin.save_model(self.request, subscribe_list_2, None, None)
        self.assertEqual(
            SubscribeList.objects.count(), start_unisender_count + 2)
        self.assertTrue(subscribe_list_2.create_list.called)
        self.assertEqual(subscribe_list_2.create_list.call_count, 1)

    def test_delete_model(self):
        start_unisender_count = SubscribeList.objects.count()
        subscribe_list = SubscribeList.objects.create(title='test')
        subscribe_list.delete_list = Mock(return_value=None)
        self.admin.delete_model(self.request, subscribe_list)
        self.assertEqual(SubscribeList.objects.count(), start_unisender_count)
        self.assertTrue(subscribe_list.delete_list.called)
        self.assertEqual(subscribe_list.delete_list.call_count, 1)


class SubscriberAdminTestCase(TestCase):

    def setUp(self):
        username = 'test_user'
        pwd = 'secret'

        self.user = User.objects.create_user(username, '', pwd)
        self.user.is_staff = True
        self.user.is_superuser = True
        self.user.save()
        self.assertTrue(self.client.login(username=username, password=pwd),
                        "Logging in user %s, pwd %s failed." % (username, pwd))
        site = AdminSite()
        self.admin = SubscriberAdmin(Subscriber, site)
        self.request = RequestFactory().get(reverse('admin:index'))

    def test_get_readonly_fields(self):
        subscribe_list = SubscribeList.objects.create(title='test', sync=True)
        self.assertIn(
            'contact',
            self.admin.get_readonly_fields(self.request, obj=subscribe_list))
        self.assertIn(
            'contact_type',
            self.admin.get_readonly_fields(self.request, subscribe_list))

    def test_open_unisender_page(self):
        """Открываем страницу подписчиков в админке"""
        response = self.client.get(
            reverse('admin:unisender_subscriber_changelist'))
        self.assertEqual(response.status_code, 200)

    def test_add_subscribe_list(self):
        """Добавляем подписчика через админку"""
        start_subscriber_count = Subscriber.objects.count()
        url = reverse('admin:unisender_subscriber_add')
        response = self.client.get(url)
        self.assertEqual(response.status_code, 200)
        csrf = get_csrf_token(response)
        subscribe_list = SubscribeList.objects.create(title='test')
        post_data = {'list_ids': subscribe_list.pk,
                     'csrfmiddlewaretoken': csrf,
                     'contact_type': 'email',
                     'contact': 'email@example.com',
                     'fields-TOTAL_FORMS': 0,
                     'fields-INITIAL_FORMS': 0,
                     'fields-MAX_NUM_FORMS': 1000,
                     'double_optin': '1',
                     }
        response = self.client.post(url, post_data)
        self.assertRedirects(
            response, reverse('admin:unisender_subscriber_changelist'))
        self.assertEqual(
            Subscriber.objects.count(), start_subscriber_count + 1)

    def test_update_subscribe_list(self):
        """Редактируем получателя"""
        subscriber = Subscriber.objects.create(contact='email@example.com')
        url = reverse(
            'admin:unisender_subscriber_change', args=(subscriber.pk,))
        response = self.client.get(url)
        self.assertEqual(response.status_code, 200)
        csrf = get_csrf_token(response)
        subscribe_list = SubscribeList.objects.create(title='test')
        post_data = {'list_ids': subscribe_list.pk,
                     'csrfmiddlewaretoken': csrf,
                     'contact_type': 'email',
                     'contact': 'email@example.com',
                     'fields-TOTAL_FORMS': 0,
                     'fields-INITIAL_FORMS': 0,
                     'fields-MAX_NUM_FORMS': 1000,
                     'double_optin': 1,
                     }
        response = self.client.post(url, post_data)
        self.assertRedirects(
            response, reverse('admin:unisender_subscriber_changelist'))
        self.assertTrue(Subscriber.objects.filter(
            contact='email@example.com').exists())

    def test_delete_subscribe_list(self):
        """Удаляем получателя"""
        start_unisender_count = Subscriber.objects.count()
        subscriber = Subscriber.objects.create(contact='email@example.com')
        url = reverse(
            'admin:unisender_subscriber_delete', args=(subscriber.pk,))
        response = self.client.get(url)
        self.assertEqual(response.status_code, 200)
        csrf = get_csrf_token(response)
        post_data = {'csrfmiddlewaretoken': csrf}
        response = self.client.post(url, post_data)
        self.assertRedirects(
            response, reverse('admin:unisender_subscriber_changelist'))
        self.assertEqual(Subscriber.objects.count(), start_unisender_count)

    def test_admin_get_actions(self):
        actions = self.admin.get_actions(self.request)
        self.assertEqual(len(actions.keys()), 1)
        self.assertIn('delete_selected_subscribers', actions)

    def test_delete_selected_subscribers(self):
        start_unisender_count = Subscriber.objects.count()
        Subscriber.exclude = Mock(return_value=None)
        for x in xrange(3):
            Subscriber.objects.create(contact='email@example.com')
        queryset = Subscriber.objects.all()
        self.admin.delete_selected_subscribers(self.request, queryset)
        self.assertTrue(Subscriber.exclude.called)
        self.assertEqual(Subscriber.exclude.call_count, 3)
        self.assertEqual(Subscriber.objects.count(), start_unisender_count)

    def test_save_model(self):
        start_unisender_count = Subscriber.objects.count()
        subscriber = Subscriber.objects.create(contact='email@example.com')
        subscriber.subscribe = Mock(return_value=None)
        self.admin.save_model(self.request, subscriber, None, None)
        self.assertEqual(Subscriber.objects.count(), start_unisender_count + 1)
        self.assertTrue(subscriber.subscribe.called)
        self.assertEqual(subscriber.subscribe.call_count, 1)

    def test_delete_model(self):
        start_unisender_count = Subscriber.objects.count()
        subscriber = Subscriber.objects.create(contact='email@example.com')
        subscriber.exclude = Mock(return_value=None)
        self.admin.delete_model(self.request, subscriber)
        self.assertEqual(Subscriber.objects.count(), start_unisender_count)
        self.assertTrue(subscriber.exclude.called)
        self.assertEqual(subscriber.exclude.call_count, 1)


class EmailMessageAdminTestCase(TestCase):

    def setUp(self):
        username = 'test_user'
        pwd = 'secret'

        self.user = User.objects.create_user(username, '', pwd)
        self.user.is_staff = True
        self.user.is_superuser = True
        self.user.save()
        self.assertTrue(self.client.login(username=username, password=pwd),
                        "Logging in user %s, pwd %s failed." % (username, pwd))
        site = AdminSite()
        self.admin = EmailMessageAdmin(EmailMessage, site)
        self.request = RequestFactory().get(reverse('admin:index'))
        self.request.user = self.user
        setattr(self.request, 'session', 'session')
        messages = FallbackStorage(self.request)
        setattr(self.request, '_messages', messages)

    def test_open_unisender_page(self):
        """Открываем страницу email сообщений в админке"""
        response = self.client.get(
            reverse('admin:unisender_emailmessage_changelist'))
        self.assertEqual(response.status_code, 200)
        self.assertIn('unisender_site', response.context)
        self.assertEqual(response.context['unisender_site'], EMAIL_MESSAGES_LIST)

    def test_add_email_message(self):
        """Добавляем email сообщение через админку"""
        start_subscriber_count = EmailMessage.objects.count()
        url = reverse('admin:unisender_emailmessage_add')
        response = self.client.get(url)
        self.assertEqual(response.status_code, 200)
        csrf = get_csrf_token(response)
        subscribe_list = SubscribeList.objects.create(title='test')
        post_data = {'list_id': subscribe_list.pk,
                     'csrfmiddlewaretoken': csrf,
                     'sender_name': 'test',
                     'sender_email': 'mail@example.com',
                     'subject': 'test',
                     'body': 'test',
                     'lang': 'ru',
                     'generate_text': 1,
                     'wrap_type': 'skip',
                     'double_optin': 1,
                     'series_time': '11:00',
                     'attachments-TOTAL_FORMS': 0,
                     'attachments-INITIAL_FORMS': 0,
                     'attachments-MAX_NUM_FORMS': 1000,
                     }
        response = self.client.post(url, post_data)
        self.assertRedirects(
            response, reverse('admin:unisender_emailmessage_changelist'))
        self.assertEqual(
            EmailMessage.objects.count(), start_subscriber_count + 1)

    def test_update_email_message(self):
        """Редактируем сообщение через админку"""
        subscribe_list = SubscribeList.objects.create(title='test')
        email = EmailMessage.objects.create(
            sender_name='test', sender_email='mail@example.com', subject='test',
            body='test', lang='ru', generate_text=1, wrap_type='skip',
            list_id=subscribe_list, unisender_id=1, sync=True
        )
        url = reverse('admin:unisender_emailmessage_change', args=(email.pk,))
        response = self.client.get(url)
        self.assertEqual(response.status_code, 200)
        self.assertIn('unisender_site', response.context)
        self.assertEqual(response.context['unisender_site'],
                         EMAIL_MESSAGES_DETAIL + str(email.unisender_id))

    def test_delete_email_message(self):
        """Удаляем email сообщение"""
        start_unisender_count = EmailMessage.objects.count()
        subscribe_list = SubscribeList.objects.create(title='test')
        message = EmailMessage.objects.create(
            sender_name='test', sender_email='mail@example.com', subject='test',
            body='test', lang='ru', generate_text=1, wrap_type='skip',
            list_id=subscribe_list
        )
        url = reverse(
            'admin:unisender_emailmessage_delete', args=(message.pk,))
        response = self.client.get(url)
        self.assertEqual(response.status_code, 200)
        csrf = get_csrf_token(response)
        post_data = {'csrfmiddlewaretoken': csrf}
        response = self.client.post(url, post_data)
        self.assertRedirects(
            response, reverse('admin:unisender_emailmessage_changelist'))
        self.assertEqual(EmailMessage.objects.count(), start_unisender_count)

    def test_inlines_readonly(self):
        '''Проверяем что вложения становятся readonly'''
        self.admin.add_view(self.request)
        self.assertListEqual(self.admin.inlines, [AttachmentInline, ])
        subscribe_list = SubscribeList.objects.create(title='test')
        message = EmailMessage.objects.create(
            sender_name='test', sender_email='mail@example.com', subject='test',
            body='test', lang='ru', generate_text=1, wrap_type='skip',
            list_id=subscribe_list
        )
        self.admin.change_view(self.request, str(message.pk))
        self.assertListEqual(self.admin.inlines, [AttachmentInlineReadOnly, ])

    def test_admin_get_actions(self):
        actions = self.admin.get_actions(self.request)
        self.assertEqual(len(actions.keys()), 1)
        self.assertIn('delete_selected_emails', actions)

    def test_delete_selected_emails(self):
        start_unisender_count = EmailMessage.objects.count()
        EmailMessage.delete_message = Mock(return_value=None)
        subscribe_list = SubscribeList.objects.create(title='test')
        for x in xrange(3):
            EmailMessage.objects.create(
                sender_name='test', sender_email='mail@example.com', subject='test',
                body='test', lang='ru', generate_text=1, wrap_type='skip',
                list_id=subscribe_list
            )
        queryset = EmailMessage.objects.all()
        self.admin.delete_selected_emails(self.request, queryset)
        self.assertTrue(EmailMessage.delete_message.called)
        self.assertEqual(EmailMessage.delete_message.call_count, 3)
        self.assertEqual(EmailMessage.objects.count(), start_unisender_count)

    def test_save_model(self):
        start_unisender_count = Subscriber.objects.count()
        subscribe_list = SubscribeList.objects.create(title='test')
        message = EmailMessage.objects.create(
            sender_name='test', sender_email='mail@example.com', subject='test',
            body='test', lang='ru', generate_text=1, wrap_type='skip',
            list_id=subscribe_list
        )
        message.create_email_message = Mock(return_value=None)
        self.admin.response_add(self.request, message)
        self.assertEqual(
            EmailMessage.objects.count(), start_unisender_count + 1)
        self.assertTrue(message.create_email_message.called)
        self.assertEqual(message.create_email_message.call_count, 1)

        message_2 = EmailMessage.objects.create(
            sender_name='test', sender_email='mail@example.com', subject='test',
            body='test', lang='ru', generate_text=1, wrap_type='skip',
            list_id=subscribe_list, unisender_id=1
        )
        message_2.create_email_message = Mock(return_value=None)
        self.admin.response_add(self.request, message_2)
        self.assertEqual(EmailMessage.objects.count(),
                         start_unisender_count + 2)
        self.assertFalse(message_2.create_email_message.called)

    def test_delete_model(self):
        start_unisender_count = EmailMessage.objects.count()
        subscribe_list = SubscribeList.objects.create(title='test')
        message = EmailMessage.objects.create(
            sender_name='test', sender_email='mail@example.com', subject='test',
            body='test', lang='ru', generate_text=1, wrap_type='skip',
            list_id=subscribe_list
        )
        message.delete_message = Mock(return_value=None)
        self.admin.delete_model(self.request, message)
        self.assertEqual(EmailMessage.objects.count(), start_unisender_count)
        self.assertTrue(message.delete_message.called)
        self.assertEqual(message.delete_message.call_count, 1)

    def test_get_readonly_fields(self):
        readonly_fields_initial = ['unisender_id', 'sync', 'get_last_error']
        self.assertListEqual(
            readonly_fields_initial,
            self.admin.get_readonly_fields(self.request))
        subscribe_list = SubscribeList.objects.create(title='test')
        message = EmailMessage.objects.create(
            sender_name='test', sender_email='mail@example.com', subject='test',
            body='test', lang='ru', generate_text=1, wrap_type='skip',
            list_id=subscribe_list, sync=True
        )
        readonly_fields_all = readonly_fields_initial
        readonly_fields_all.extend([
            'sender_name', 'sender_email', 'subject', 'body', 'list_id', 'lang',
            'text_body', 'generate_text', 'wrap_type', 'categories', 'tag',
            'read_only_body', 'series_day', 'last_error', 'series_time', 'id'])
        self.assertItemsEqual(
            readonly_fields_all,
            self.admin.get_readonly_fields(self.request, message))

    def test_get_fieldsets(self):
        fieldsets_initial = [
            [u'Unisender',
             {'fields': ['unisender_id', 'sync', 'get_last_error']}],
            [u'Сообщение',
             {'fields': ['sender_name', 'sender_email', 'subject', 'body',
                         'list_id', 'lang', 'text_body', 'generate_text',
                         'wrap_type', 'categories', 'tag']}],
            [u'Автоматическая отправка',
             {'fields': ['series_day', 'series_time']}]]
        self.assertItemsEqual(
            fieldsets_initial,
            self.admin.get_fieldsets(self.request))
        subscribe_list = SubscribeList.objects.create(title='test')
        message = EmailMessage.objects.create(
            sender_name='test', sender_email='mail@example.com', subject='test',
            body='test', lang='ru', generate_text=1, wrap_type='skip',
            list_id=subscribe_list, unisender_id=1
        )
        fieldsets_all = [
            [u'Unisender',
             {'fields': ['unisender_id', 'sync', 'get_last_error']}],
            [u'Сообщение',
             {'fields': ['sender_name', 'sender_email', 'subject',
                         'read_only_body',
                         'list_id', 'lang', 'text_body', 'generate_text',
                         'wrap_type', 'categories', 'tag']}],
            [u'Автоматическая отправка',
             {'fields': ['series_day', 'series_time']}]]
        self.assertItemsEqual(
            fieldsets_all,
            self.admin.get_fieldsets(self.request, message))


class CampaignAdminTestCase(TestCase):

    def setUp(self):
        username = 'test_user'
        pwd = 'secret'

        self.user = User.objects.create_user(username, '', pwd)
        self.user.is_staff = True
        self.user.is_superuser = True
        self.user.save()
        self.assertTrue(self.client.login(username=username, password=pwd),
                        "Logging in user %s, pwd %s failed." % (username, pwd))
        site = AdminSite()
        self.admin = CampaignAdmin(Campaign, site)
        self.request = RequestFactory().get(reverse('admin:index'))
        subscribe_list = SubscribeList.objects.create(title='test')
        self.message = EmailMessage.objects.create(
            sender_name='test', sender_email='mail@example.com', subject='test',
            body='test', lang='ru', generate_text=1, wrap_type='skip',
            list_id=subscribe_list, sync=True
        )
        self.request.user = self.user

    def test_open_unisender_page(self):
        """Открываем страницу рассылки в админке"""
        response = self.client.get(
            reverse('admin:unisender_campaign_changelist'))
        self.assertEqual(response.status_code, 200)
        self.assertIn('unisender_site', response.context)
        self.assertEqual(response.context['unisender_site'], CAMPAIGN_LIST)

    def test_add_campaign(self):
        """Добавляем рассылку через админку"""
        start_subscriber_count = Campaign.objects.count()
        url = reverse('admin:unisender_campaign_add')
        response = self.client.get(url)
        self.assertEqual(response.status_code, 200)
        csrf = get_csrf_token(response)
        post_data = {'csrfmiddlewaretoken': csrf,
                     'name': 'test',
                     'email_message': self.message.pk,
                     'track_read': 0,
                     'track_links': 0,
                     'track_ga': 0,
                     'visited_links-TOTAL_FORMS': 0,
                     'visited_links-INITIAL_FORMS': 0,
                     'visited_links-MAX_NUM_FORMS': 0,
                     }
        response = self.client.post(url, post_data)
        self.assertRedirects(
            response, reverse('admin:unisender_campaign_changelist'))
        self.assertEqual(Campaign.objects.count(), start_subscriber_count + 1)

    def test_update_campaign(self):
        """Редактируем рассылку"""
        campaign = Campaign.objects.create(
            name='test', email_message=self.message, unisender_id=1)
        url = reverse(
            'admin:unisender_campaign_change', args=(campaign.pk,))
        response = self.client.get(url)
        self.assertEqual(response.status_code, 200)
        self.assertIn('unisender_site', response.context)
        self.assertEqual(response.context['unisender_site'],
                         CAMPAIGN_DETAIL + str(campaign.unisender_id))
        csrf = get_csrf_token(response)
        post_data = {'csrfmiddlewaretoken': csrf,
                     'name': 'test-2',
                     'email_message': self.message.pk,
                     'track_read': 0,
                     'track_links': 0,
                     'track_ga': 0,
                     'visited_links-TOTAL_FORMS': 0,
                     'visited_links-INITIAL_FORMS': 0,
                     'visited_links-MAX_NUM_FORMS': 0,
                     }
        response = self.client.post(url, post_data)
        self.assertRedirects(
            response, reverse('admin:unisender_campaign_changelist'))
        self.assertTrue(Campaign.objects.filter(
            name='test-2').exists())

    def test_delete_campaign(self):
        """Удаляем рассылку"""
        start_unisender_count = Campaign.objects.count()
        campaign = Campaign.objects.create(
            name='test', email_message=self.message)
        url = reverse('admin:unisender_campaign_delete', args=(campaign.pk,))
        response = self.client.get(url)
        self.assertEqual(response.status_code, 200)
        csrf = get_csrf_token(response)
        post_data = {'csrfmiddlewaretoken': csrf}
        response = self.client.post(url, post_data)
        self.assertRedirects(
            response, reverse('admin:unisender_campaign_changelist'))
        self.assertEqual(Campaign.objects.count(), start_unisender_count)

    def test_admin_get_actions(self):
        actions = self.admin.get_actions(self.request)
        self.assertEqual(len(actions.keys()), 1)
        self.assertIn('delete_selected_campaigns', actions)

    @patch.object(messages, 'warning', mock_messages)
    def test_delete_selected_emails(self):
        start_unisender_count = Campaign.objects.count()
        for x in xrange(3):
            Campaign.objects.create(
                name='test', email_message=self.message)
        queryset = Campaign.objects.all()
        self.admin.delete_selected_campaigns(self.request, queryset)
        self.assertEqual(Campaign.objects.count(), start_unisender_count)

    def test_save_model(self):
        start_unisender_count = Campaign.objects.count()
        campaign = Campaign.objects.create(
            name='test', email_message=self.message, unisender_id=1)
        campaign.create_campaign = Mock(return_value=None)
        self.admin.save_model(self.request, campaign, None, None)
        self.assertEqual(
            EmailMessage.objects.count(), start_unisender_count + 1)
        self.assertFalse(campaign.create_campaign.called)

        campaign.unisender_id = None
        campaign.save()
        self.admin.save_model(self.request, campaign, None, None)
        self.assertEqual(
            EmailMessage.objects.count(), start_unisender_count + 1)
        self.assertTrue(campaign.create_campaign.called)
        self.assertEqual(campaign.create_campaign.call_count, 1)

    @patch.object(messages, 'warning', mock_messages)
    def test_delete_model(self):
        start_unisender_count = Campaign.objects.count()
        campaign = Campaign.objects.create(
            name='test', email_message=self.message)
        self.admin.delete_model(self.request, campaign)
        self.assertEqual(Campaign.objects.count(), start_unisender_count)

    def test_get_readonly_fields(self):
        readonly_fields_initial = ['unisender_id', 'sync', 'get_last_error']
        self.assertListEqual(
            readonly_fields_initial,
            self.admin.get_readonly_fields(self.request))
        campaign = Campaign.objects.create(
            name='test', email_message=self.message, sync=True)
        readonly_fields_all = ([
            'track_links', 'err_resend', 'track_ga', 'err_skip_letter',
            'last_error', 'ok_delivered', 'sync', 'track_read',
            'err_spam_rejected', 'ok_link_visited', 'payment_limit', 'total',
            'err_dest_invalid', u'id', 'err_spam_skipped', 'err_spam_folder',
            'err_internal', 'err_user_unknown', 'err_user_inactive',
            'err_will_retry', 'ok_unsubscribed', 'status', 'err_not_available',
            'start_time', 'unisender_id', 'not_sent', 'ok_read',
            'err_domain_inactive', 'err_unsubscribed', 'err_lost', 'last_check',
            'err_spam_retry', 'name', 'err_src_invalid', 'contacts',
            'err_not_allowed', 'err_delivery_failed', 'email_message',
            'err_mailbox_full', 'ok_spam_folder', 'get_last_error',
            'get_error_count', 'get_success_count']
        )
        self.assertItemsEqual(
            readonly_fields_all,
            self.admin.get_readonly_fields(self.request, campaign))

    def test_get_fieldsets(self):
        fieldsets_initial = [
            [u'Unisender', {
             'fields': ['unisender_id', 'sync', 'get_last_error']
             }],
            (u'Рассылка', {
                'fields': ['name', 'email_message', 'start_time',
                           'track_read', 'track_links', 'contacts', 'track_ga',
                           'payment_limit', ]
            })]
        self.assertItemsEqual(
            fieldsets_initial,
            self.admin.get_fieldsets(self.request))
        campaign = Campaign.objects.create(
            name='test', email_message=self.message, sync=True)
        fieldsets_all = fieldsets_initial + [
                (u'Статус', {
                 'fields': ['status', 'start_time', ]
                 }),
                (u'Краткая информация', {
                 'fields': ['get_error_count', 'get_success_count']
                 }),
                (u'Подробная информация', {
                 'fields': ['not_sent', 'ok_delivered', 'ok_read',
                            'ok_spam_folder', 'ok_link_visited',
                            'ok_unsubscribed',
                            'err_user_unknown', 'err_user_inactive',
                            'err_mailbox_full', 'err_spam_rejected',
                            'err_spam_folder', 'err_delivery_failed',
                            'err_will_retry', 'err_resend', 'err_not_allowed',
                            'err_domain_inactive', 'err_unsubscribed',
                            'err_skip_letter', 'err_spam_retry',
                            'err_src_invalid', 'err_dest_invalid',
                            'err_not_available', 'err_lost', 'err_internal', ]
                 })]
        self.assertItemsEqual(
            fieldsets_all,
            self.admin.get_fieldsets(self.request, campaign))

    def test_get_urls(self):
        campaign = Campaign.objects.create(
            name='test', email_message=self.message, sync=True)
        self.assertEqual(
            reverse('admin:unisender_campaign_get_statistic',
                    kwargs={'pk': campaign.pk}),
            '/admin/unisender/campaign/%s/get_statistic/' % campaign.pk)



# -*- coding: utf-8 -*-
from django.test import TestCase

from mock import patch

from unisender.models import Tag, Field, SubscribeList, Campaign


class TagManagerTestCase(TestCase):

    tag_result = {
        'result': [
        {'id': 1, 'name': 'test 1'},
        {'id': 2, 'name': 'test 2'}
        ]
    }

    def setUp(self):
        self.manager = Tag.unisender
        with patch('unisender.managers.UnisenderManager.api') as mock:
            self.manager.api = mock.return_value
            self.manager.api.getTags.return_value = self.tag_result

    def test__get_tags(self):
        # simple test
        tags = Tag.unisender.get_tags()
        resulted_tags = {
            'result': [
                {'id': 1, 'name': 'test 1'},
                {'id': 2, 'name': 'test 2'}
            ]
        }
        self.assertEquals(tags, resulted_tags)

    def test__get_and_update_tags(self):
        tag_1 = Tag.objects.create(name='test 1')
        self.assertFalse(tag_1.sync)
        self.manager.get_and_update_tags()
        tag_1 = Tag.objects.get(unisender_id=1)
        self.assertTrue(tag_1.sync)
        tag_2 = Tag.objects.get(unisender_id=2)
        self.assertEquals(tag_2.name, 'test 2')
        self.assertTrue(tag_2.sync)


class FieldManagerTestCase(TestCase):

    field_result = {
        'result': [
        {'id': 1, 'name': 'test 1', 'type':
         'string', 'is_visible': 1, 'view_pos': 1},
        {'id': 2, 'name': 'test 2', 'type':
         'text', 'is_visible': 1, 'view_pos': 3}
        ]
    }

    def setUp(self):
        self.manager = Field.unisender
        with patch('unisender.managers.UnisenderManager.api') as mock:
            self.manager.api = mock.return_value
            self.manager.api.getFields.return_value = self.field_result

    def test__get_fields(self):
        # simple test
        fields = self.manager.get_fields()
        resulted_fields = {
            'result': [
                {'id': 1, 'name': 'test 1', 'type':
                 'string', 'is_visible': 1, 'view_pos': 1},
                {'id': 2, 'name': 'test 2', 'type':
                 'text', 'is_visible': 1, 'view_pos': 3}
            ]
        }
        self.assertEquals(fields, resulted_fields)

    def test__get_and_update_fields(self):
        Field.objects.create(
            name='test 1', field_type='string', sort=3)
        self.manager.get_and_update_fields()
        field_1 = Field.objects.get(name='test 1')
        self.assertEquals(field_1.field_type, 'string')
        self.assertEquals(field_1.sort, 1)
        self.assertTrue(field_1.sync)
        self.assertTrue(field_1.visible)

        field_2 = Field.objects.get(name='test 2')
        self.assertEquals(field_2.field_type, 'text')
        self.assertTrue(field_2.sync)
        self.assertTrue(field_2.visible)
        self.assertEquals(field_2.sort, 3)


class ListManagerTestCase(TestCase):

    list_get_result = {
        'result': [
            {'id': 1, 'title': 'test 1'},
            {'id': 2, 'title': 'test 2'},
        ]
    }

    def setUp(self):
        self.manager = SubscribeList.unisender
        with patch('unisender.managers.UnisenderManager.api') as mock:
            self.manager.api = mock.return_value
            self.manager.api.getLists.return_value = self.list_get_result

    def test__get_subscribe_lists(self):
        # simple test
        subscribe_list = self.manager.get_lists()
        resulted_list = {
            'result': [
                {'id': 1, 'title': 'test 1'},
                {'id': 2, 'title': 'test 2'}
            ]
        }
        self.assertEquals(subscribe_list, resulted_list)

    def test__get_and_update_subscribe_lists(self):
        list_1 = SubscribeList.objects.create(title='test 1')
        self.assertFalse(list_1.sync)
        self.manager.get_and_update_lists()
        list_1 = SubscribeList.objects.get(title='test 1')
        self.assertTrue(list_1.sync)
        list_2 = SubscribeList.objects.get(title='test 2')
        self.assertTrue(list_2.sync)


class CampaignManagerTestCase(TestCase):

    list_get_campaings = {
        'result': [
            {'id': 1, 'start_time':
             '2011-08-01 19:30:00', 'status': 'completed'},
            {'id': 2, 'start_time':
                '2011-08-10 19:30:00', 'status': 'analysed'},
        ]
    }

    def setUp(self):
        self.manager = Campaign.unisender
        with patch('unisender.managers.UnisenderManager.api') as mock:
            self.manager.api = mock.return_value
            self.manager.api.getCampaigns.return_value = self.list_get_campaings

    def test__get_campaings(self):
        # simple test
        subscribe_list = self.manager.get_campaigns()
        resulted_list = {
            'result': [
            {'id': 1, 'start_time':
             '2011-08-01 19:30:00', 'status': 'completed'},
            {'id': 2, 'start_time':
                '2011-08-10 19:30:00', 'status': 'analysed'},
            ]
        }
        self.assertEquals(subscribe_list, resulted_list)

    def test__get_campaings_time_filter(self):
        # TODO
        pass



# -*- coding: utf-8 -*-
# django
from django.core.urlresolvers import reverse_lazy
from django.shortcuts import get_object_or_404
from django.views.generic.base import RedirectView

#app
from unisender.models import Campaign, Tag, Field, SubscribeList


class GetCampaignStatistic(RedirectView):
    permanent = False
    pattern_name = 'admin:unisender_campaign_changelist'

    def get_redirect_url(self, *args, **kwargs):
        campaign = get_object_or_404(Campaign, pk=self.kwargs['pk'])
        campaign.get_campaign_status()
        if campaign.get_last_error() is None:
            campaign.get_campaign_agregate_status()
            campaign.get_visited_links()
        campaign.save()
        self.url = reverse_lazy(
            'admin:unisender_campaign_change', args=(self.kwargs['pk']))
        return super(GetCampaignStatistic, self).get_redirect_url(
            *args, **kwargs)


class GetTags(RedirectView):
    permanent = False
    pattern_name = 'admin:unisender_tag_changelist'

    def get_redirect_url(self, *args, **kwargs):
        Tag.unisender.get_and_update_tags(self.request)
        return super(GetTags, self).get_redirect_url(*args, **kwargs)


class GetFields(RedirectView):
    permanent = False
    pattern_name = 'admin:unisender_field_changelist'

    def get_redirect_url(self, *args, **kwargs):
        Field.unisender.get_and_update_fields(self.request)
        return super(GetFields, self).get_redirect_url(*args, **kwargs)


class GetLists(RedirectView):
    permanent = False
    pattern_name = 'admin:unisender_subscribelist_changelist'

    def get_redirect_url(self, *args, **kwargs):
        SubscribeList.unisender.get_and_update_lists(self.request)
        return super(GetLists, self).get_redirect_url(*args, **kwargs)


class GetCampaigns(RedirectView):
    permanent = False
    pattern_name = 'admin:unisender_campaign_changelist'

    def get_redirect_url(self, *args, **kwargs):
        Campaign.unisender.get_and_update_campaigns(request=self.request)
        return super(GetCampaigns, self).get_redirect_url(*args, **kwargs)



# -*- coding: utf-8 -*-


def unisender_test_api(UnisenderModel):
    class UnisenderMockAPI(object):

        def createField(self, **kwargs):
            return {'result': {'id': 1}}

        def updateField(self, **kwargs):
            return {'id': 1}

        def deleteField(self, **kwargs):
            return {'result': {}}

        def deleteList(self, **kwargs):
            return {'result': {}}

        def updateList(self, **kwargs):
            return {'result': {}}

        def createList(self, **kwargs):
            return {'result': {'id': 1}}

        def subscribe(self, **kwargs):
            return {'result': {'person_id': 1}}

        def unsubscribe(self, **kwargs):
            return {'result': {}}

        def exclude(self, **kwargs):
            return {'result': {}}

        def deleteMessage(self, **kwargs):
            return {'result': {}}

        def createEmailMessage(self, **kwargs):
            return {'result': {'message_id': 1}}

        def createCampaign(self, **kwargs):
            return {'result': {'campaign_id': 1, 'status': 'scheduled',
                               'count': 2}}

        def getCampaignStatus(self, **kwargs):
            return {'result': {'status': 'completed',
                               'creation_time': '2011-09-21 19:47:31',
                               'start_time': '2011-09-21 20:00:00'}}

        def getCampaignAggregateStats(self, **kwargs):
            return {'result': {'total': 241, 'data': {
                'ok_read': 239, 'err_will_retry': 2}}
            }

        def getVisitedLinks(self, **kwargs):
            return {
                'result': {
                    'fields': ['email', 'url', 'request_time', 'ip', 'count'],
                    'data': [
                        ['one@gmail.com', 'http://yandex.ru',
                            '2011-01-27 09:38:01', '127.0.0.1', '1'],
                        ['one@gmail.com', 'http://yandex.ru',
                            '2011-01-27 09:38:01', '127.0.0.1', '1'],
                        ['two@hotmail.com', 'http://google.com',
                         '2011-01-27 09:38:02', '127.0.0.1', '2'],
                        ['three@yandex.ru', 'http://gmail.com',
                         '2011-01-27 09:38:03', '127.0.0.1', '3'],
                    ]
                }
            }

        def updateOptInEmail(self, **kwargs):
            return {}

    return UnisenderMockAPI()


def unisender_test_api_errors(UnisenderModel):

    class UnisenderMockAPI(object):
        error_result = {'result': '', 'error': 'invalid_arg'}

        def createField(self, **kwargs):
            return self.error_result

        def updateField(self, **kwargs):
            return self.error_result

        def deleteField(self, **kwargs):
            return self.error_result

        def deleteList(self, **kwargs):
            return self.error_result

        def updateList(self, **kwargs):
            return self.error_result

        def createList(self, **kwargs):
            return self.error_result

        def subscribe(self, **kwargs):
            return self.error_result

        def unsubscribe(self, **kwargs):
            return self.error_result

        def exclude(self, **kwargs):
            return self.error_result

        def deleteMessage(self, **kwargs):
            return self.error_result

        def createEmailMessage(self, **kwargs):
            return self.error_result

        def createCampaign(self, **kwargs):
            return self.error_result

        def getCampaignStatus(self, **kwargs):
            return self.error_result

        def getCampaignAggregateStats(self, **kwargs):
            return self.error_result

        def getVisitedLinks(self, **kwargs):
            return self.error_result

        def updateOptInEmail(self, **kwargs):
            return self.error_result

    return UnisenderMockAPI()


def unisender_test_api_correct_values(UnisenderModel):

    class UnisenderMockAPI(object):

        def all_requirement_fields_present(self, requirement_fields, kwargs):
            for item in requirement_fields:
                if item not in kwargs:
                    raise NameError(
                        'В передаваемых значениях отсутствует обязательное поле < %s >' % item)

        def not_documented_fields_not_present(self, requirement_fields, fields, kwargs):
            all_fields = requirement_fields + fields
            for item in kwargs.keys():
                if item not in all_fields:
                    raise NameError(
                        'В передаваемых значениях присутствует поле < %s > котрое не указано в документации' % item)

        def createField(self, **kwargs):
            requirement_fields = ['name', 'type']
            fields = ['is_visible', 'view_pos']
            self.all_requirement_fields_present(requirement_fields, kwargs)
            self.not_documented_fields_not_present(
                requirement_fields, fields, kwargs)
            return {}

        def updateField(self, **kwargs):
            requirement_fields = ['id']
            fields = ['name', 'type', 'is_visible', 'view_pos', ]
            self.all_requirement_fields_present(requirement_fields, kwargs)
            self.not_documented_fields_not_present(
                requirement_fields, fields, kwargs)
            return {}

        def deleteField(self, **kwargs):
            requirement_fields = ['id']
            self.all_requirement_fields_present(requirement_fields, kwargs)
            self.not_documented_fields_not_present(
                requirement_fields, [], kwargs)
            return {}

        def deleteList(self, **kwargs):
            requirement_fields = ['list_id']
            self.all_requirement_fields_present(requirement_fields, kwargs)
            self.not_documented_fields_not_present(
                requirement_fields, [], kwargs)
            return {}

        def updateList(self, **kwargs):
            requirement_fields = ['list_id']
            fields = ['title', 'before_subscribe_url', 'after_subscribe_url']
            self.all_requirement_fields_present(requirement_fields, kwargs)
            self.not_documented_fields_not_present(
                requirement_fields, fields, kwargs)
            return {}

        def createList(self, **kwargs):
            requirement_fields = ['title']
            fields = ['before_subscribe_url', 'after_subscribe_url']
            self.all_requirement_fields_present(requirement_fields, kwargs)
            self.not_documented_fields_not_present(
                requirement_fields, fields, kwargs)
            return {}

        def subscribe(self, **kwargs):
            requirement_fields = ['list_ids', 'fields']
            fields = ['tags', 'request_ip', 'request_time ', 'double_optin',
                      'confirm_ip', 'confirm_time', 'overwrite']
            self.all_requirement_fields_present(requirement_fields, kwargs)
            self.not_documented_fields_not_present(
                requirement_fields, fields, kwargs)
            return {}

        def unsubscribe(self, **kwargs):
            requirement_fields = ['contact_type', 'contact']
            fields = ['list_ids']
            self.all_requirement_fields_present(requirement_fields, kwargs)
            self.not_documented_fields_not_present(
                requirement_fields, fields, kwargs)
            return {}

        def exclude(self, **kwargs):
            requirement_fields = ['contact_type', 'contact']
            fields = ['list_ids']
            self.all_requirement_fields_present(requirement_fields, kwargs)
            self.not_documented_fields_not_present(
                requirement_fields, fields, kwargs)
            return {}

        def deleteMessage(self, **kwargs):
            requirement_fields = ['message_id']
            self.all_requirement_fields_present(requirement_fields, kwargs)
            self.not_documented_fields_not_present(
                requirement_fields, [], kwargs)
            return {}

        def createEmailMessage(self, **kwargs):
            requirement_fields = ['sender_name', 'sender_email', 'subject',
                                  'body', 'list_id']
            fields = ['text_body', 'generate_text', 'tag', 'attachments',
                      'lang', 'series_day', 'series_time', 'wrap_type',
                      'categories']
            self.all_requirement_fields_present(requirement_fields, kwargs)
            self.not_documented_fields_not_present(
                requirement_fields, fields, kwargs)
            return {}

        def createCampaign(self, **kwargs):
            fields = [
                'message_id', 'start_time', 'timezone', 'track_read',
                'track_links', 'contacts', 'contacts_url', 'defer', 'track_ga',
                'payment_limit', 'payment_currency', 'ga_medium', 'ga_source',
                'ga_campaign', 'ga_content', 'ga_term']
            self.not_documented_fields_not_present([], fields, kwargs)
            return {}

        def getCampaignStatus(self, **kwargs):
            requirement_fields = ['campaign_id']
            self.all_requirement_fields_present(requirement_fields, kwargs)
            self.not_documented_fields_not_present(
                requirement_fields, [], kwargs)
            return {}

        def getCampaignAggregateStats(self, **kwargs):
            requirement_fields = ['campaign_id']
            self.all_requirement_fields_present(requirement_fields, kwargs)
            self.not_documented_fields_not_present(
                requirement_fields, [], kwargs)
            return {}

        def getVisitedLinks(self, **kwargs):
            requirement_fields = ['campaign_id',]
            fields = ['group',]
            self.all_requirement_fields_present(requirement_fields, kwargs)
            self.not_documented_fields_not_present(
                requirement_fields, fields, kwargs)
            return {}

        def updateOptInEmail(self, **kwargs):
            requirement_fields = ['sender_name', 'sender_email', 'subject',
                                  'body', 'list_id',]
            self.all_requirement_fields_present(requirement_fields, kwargs)
            self.not_documented_fields_not_present(
                requirement_fields, [], kwargs)
            return {}

    return UnisenderMockAPI()


def mock_messages(request, message):
    return None



# -*- coding: utf-8 -*-
from south.utils import datetime_utils as datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models


class Migration(SchemaMigration):

    def forwards(self, orm):
        # Adding model 'Tag'
        db.create_table(u'unisender_tag', (
            (u'id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('unisender_id', self.gf('django.db.models.fields.CharField')(max_length=255, null=True, blank=True)),
            ('last_error', self.gf('django.db.models.fields.CharField')(max_length=255, null=True, blank=True)),
            ('sync', self.gf('django.db.models.fields.BooleanField')(default=False)),
            ('name', self.gf('django.db.models.fields.CharField')(max_length=255)),
        ))
        db.send_create_signal(u'unisender', ['Tag'])

        # Adding model 'Field'
        db.create_table(u'unisender_field', (
            (u'id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('unisender_id', self.gf('django.db.models.fields.CharField')(max_length=255, null=True, blank=True)),
            ('last_error', self.gf('django.db.models.fields.CharField')(max_length=255, null=True, blank=True)),
            ('sync', self.gf('django.db.models.fields.BooleanField')(default=False)),
            ('name', self.gf('django.db.models.fields.CharField')(max_length=255)),
            ('field_type', self.gf('django.db.models.fields.CharField')(default='string', max_length=50)),
            ('visible', self.gf('django.db.models.fields.BooleanField')(default=True)),
            ('sort', self.gf('django.db.models.fields.SmallIntegerField')(default=1)),
        ))
        db.send_create_signal(u'unisender', ['Field'])

        # Adding model 'SubscribeList'
        db.create_table(u'unisender_subscribelist', (
            (u'id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('unisender_id', self.gf('django.db.models.fields.CharField')(max_length=255, null=True, blank=True)),
            ('last_error', self.gf('django.db.models.fields.CharField')(max_length=255, null=True, blank=True)),
            ('sync', self.gf('django.db.models.fields.BooleanField')(default=False)),
            ('title', self.gf('django.db.models.fields.CharField')(unique=True, max_length=255)),
            ('before_subscribe_url', self.gf('django.db.models.fields.CharField')(max_length=255, null=True, blank=True)),
            ('after_subscribe_url', self.gf('django.db.models.fields.CharField')(max_length=255, null=True, blank=True)),
        ))
        db.send_create_signal(u'unisender', ['SubscribeList'])

        # Adding model 'Subscriber'
        db.create_table(u'unisender_subscriber', (
            (u'id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('unisender_id', self.gf('django.db.models.fields.CharField')(max_length=255, null=True, blank=True)),
            ('last_error', self.gf('django.db.models.fields.CharField')(max_length=255, null=True, blank=True)),
            ('sync', self.gf('django.db.models.fields.BooleanField')(default=False)),
            ('contact_type', self.gf('django.db.models.fields.CharField')(default='email', max_length=50)),
            ('contact', self.gf('django.db.models.fields.CharField')(max_length=255)),
            ('double_optin', self.gf('django.db.models.fields.SmallIntegerField')(default=1)),
        ))
        db.send_create_signal(u'unisender', ['Subscriber'])

        # Adding M2M table for field list_ids on 'Subscriber'
        m2m_table_name = db.shorten_name(u'unisender_subscriber_list_ids')
        db.create_table(m2m_table_name, (
            ('id', models.AutoField(verbose_name='ID', primary_key=True, auto_created=True)),
            ('subscriber', models.ForeignKey(orm[u'unisender.subscriber'], null=False)),
            ('subscribelist', models.ForeignKey(orm[u'unisender.subscribelist'], null=False))
        ))
        db.create_unique(m2m_table_name, ['subscriber_id', 'subscribelist_id'])

        # Adding M2M table for field tags on 'Subscriber'
        m2m_table_name = db.shorten_name(u'unisender_subscriber_tags')
        db.create_table(m2m_table_name, (
            ('id', models.AutoField(verbose_name='ID', primary_key=True, auto_created=True)),
            ('subscriber', models.ForeignKey(orm[u'unisender.subscriber'], null=False)),
            ('tag', models.ForeignKey(orm[u'unisender.tag'], null=False))
        ))
        db.create_unique(m2m_table_name, ['subscriber_id', 'tag_id'])

        # Adding model 'SubscriberFields'
        db.create_table(u'unisender_subscriberfields', (
            (u'id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('subscriber', self.gf('django.db.models.fields.related.ForeignKey')(related_name='fields', to=orm['unisender.Subscriber'])),
            ('field', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['unisender.Field'])),
            ('value', self.gf('django.db.models.fields.CharField')(max_length=255)),
        ))
        db.send_create_signal(u'unisender', ['SubscriberFields'])

        # Adding model 'EmailMessage'
        db.create_table(u'unisender_emailmessage', (
            (u'id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('unisender_id', self.gf('django.db.models.fields.CharField')(max_length=255, null=True, blank=True)),
            ('last_error', self.gf('django.db.models.fields.CharField')(max_length=255, null=True, blank=True)),
            ('sync', self.gf('django.db.models.fields.BooleanField')(default=False)),
            ('sender_name', self.gf('django.db.models.fields.CharField')(max_length=255)),
            ('sender_email', self.gf('django.db.models.fields.CharField')(max_length=255)),
            ('subject', self.gf('django.db.models.fields.CharField')(max_length=255)),
            ('body', self.gf('tinymce_4.fields.TinyMCEModelField')()),
            ('list_id', self.gf('django.db.models.fields.related.ForeignKey')(related_name='emails', to=orm['unisender.SubscribeList'])),
            ('tag', self.gf('django.db.models.fields.related.ForeignKey')(blank=True, related_name='emails', null=True, to=orm['unisender.Tag'])),
            ('lang', self.gf('django.db.models.fields.CharField')(default='ru', max_length=50)),
            ('text_body', self.gf('django.db.models.fields.TextField')(null=True, blank=True)),
            ('generate_text', self.gf('django.db.models.fields.CharField')(default='1', max_length=50)),
            ('wrap_type', self.gf('django.db.models.fields.CharField')(default='skip', max_length=50)),
            ('categories', self.gf('django.db.models.fields.CharField')(max_length=255, null=True, blank=True)),
            ('series_day', self.gf('django.db.models.fields.PositiveSmallIntegerField')(null=True, blank=True)),
            ('series_time', self.gf('django.db.models.fields.TimeField')(default=datetime.datetime(2014, 7, 8, 0, 0))),
        ))
        db.send_create_signal(u'unisender', ['EmailMessage'])

        # Adding model 'SmsMessage'
        db.create_table(u'unisender_smsmessage', (
            (u'id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('unisender_id', self.gf('django.db.models.fields.CharField')(max_length=255, null=True, blank=True)),
            ('last_error', self.gf('django.db.models.fields.CharField')(max_length=255, null=True, blank=True)),
            ('sync', self.gf('django.db.models.fields.BooleanField')(default=False)),
        ))
        db.send_create_signal(u'unisender', ['SmsMessage'])

        # Adding model 'Campaign'
        db.create_table(u'unisender_campaign', (
            (u'id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('unisender_id', self.gf('django.db.models.fields.CharField')(max_length=255, null=True, blank=True)),
            ('last_error', self.gf('django.db.models.fields.CharField')(max_length=255, null=True, blank=True)),
            ('sync', self.gf('django.db.models.fields.BooleanField')(default=False)),
            ('name', self.gf('django.db.models.fields.CharField')(max_length=255)),
            ('email_message', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['unisender.EmailMessage'])),
            ('start_time', self.gf('django.db.models.fields.DateTimeField')(null=True, blank=True)),
            ('track_read', self.gf('django.db.models.fields.CharField')(default='0', max_length=50)),
            ('track_links', self.gf('django.db.models.fields.CharField')(default='0', max_length=50)),
            ('track_ga', self.gf('django.db.models.fields.CharField')(default='0', max_length=50)),
            ('payment_limit', self.gf('django.db.models.fields.PositiveSmallIntegerField')(null=True, blank=True)),
            ('status', self.gf('django.db.models.fields.CharField')(default=None, max_length=50, null=True, blank=True)),
            ('last_check', self.gf('django.db.models.fields.DateTimeField')(null=True, blank=True)),
            ('not_sent', self.gf('django.db.models.fields.PositiveSmallIntegerField')(default=0)),
            ('ok_delivered', self.gf('django.db.models.fields.PositiveSmallIntegerField')(default=0)),
            ('ok_read', self.gf('django.db.models.fields.PositiveSmallIntegerField')(default=0)),
            ('ok_spam_folder', self.gf('django.db.models.fields.PositiveSmallIntegerField')(default=0)),
            ('ok_link_visited', self.gf('django.db.models.fields.PositiveSmallIntegerField')(default=0)),
            ('ok_unsubscribed', self.gf('django.db.models.fields.PositiveSmallIntegerField')(default=0)),
            ('err_user_unknown', self.gf('django.db.models.fields.PositiveSmallIntegerField')(default=0)),
            ('err_user_inactive', self.gf('django.db.models.fields.PositiveSmallIntegerField')(default=0)),
            ('err_mailbox_full', self.gf('django.db.models.fields.PositiveSmallIntegerField')(default=0)),
            ('err_spam_rejected', self.gf('django.db.models.fields.PositiveSmallIntegerField')(default=0)),
            ('err_spam_folder', self.gf('django.db.models.fields.PositiveSmallIntegerField')(default=0)),
            ('err_delivery_failed', self.gf('django.db.models.fields.PositiveSmallIntegerField')(default=0)),
            ('err_will_retry', self.gf('django.db.models.fields.PositiveSmallIntegerField')(default=0)),
            ('err_resend', self.gf('django.db.models.fields.PositiveSmallIntegerField')(default=0)),
            ('err_domain_inactive', self.gf('django.db.models.fields.PositiveSmallIntegerField')(default=0)),
            ('err_skip_letter', self.gf('django.db.models.fields.PositiveSmallIntegerField')(default=0)),
            ('err_spam_skipped', self.gf('django.db.models.fields.PositiveSmallIntegerField')(default=0)),
            ('err_spam_retry', self.gf('django.db.models.fields.PositiveSmallIntegerField')(default=0)),
            ('err_unsubscribed', self.gf('django.db.models.fields.PositiveSmallIntegerField')(default=0)),
            ('err_src_invalid', self.gf('django.db.models.fields.PositiveSmallIntegerField')(default=0)),
            ('err_dest_invalid', self.gf('django.db.models.fields.PositiveSmallIntegerField')(default=0)),
            ('err_not_allowed', self.gf('django.db.models.fields.PositiveSmallIntegerField')(default=0)),
            ('err_not_available', self.gf('django.db.models.fields.PositiveSmallIntegerField')(default=0)),
            ('err_lost', self.gf('django.db.models.fields.PositiveSmallIntegerField')(default=0)),
            ('err_internal', self.gf('django.db.models.fields.PositiveSmallIntegerField')(default=0)),
            ('total', self.gf('django.db.models.fields.PositiveSmallIntegerField')(default=0)),
        ))
        db.send_create_signal(u'unisender', ['Campaign'])

        # Adding M2M table for field contacts on 'Campaign'
        m2m_table_name = db.shorten_name(u'unisender_campaign_contacts')
        db.create_table(m2m_table_name, (
            ('id', models.AutoField(verbose_name='ID', primary_key=True, auto_created=True)),
            ('campaign', models.ForeignKey(orm[u'unisender.campaign'], null=False)),
            ('subscriber', models.ForeignKey(orm[u'unisender.subscriber'], null=False))
        ))
        db.create_unique(m2m_table_name, ['campaign_id', 'subscriber_id'])


    def backwards(self, orm):
        # Deleting model 'Tag'
        db.delete_table(u'unisender_tag')

        # Deleting model 'Field'
        db.delete_table(u'unisender_field')

        # Deleting model 'SubscribeList'
        db.delete_table(u'unisender_subscribelist')

        # Deleting model 'Subscriber'
        db.delete_table(u'unisender_subscriber')

        # Removing M2M table for field list_ids on 'Subscriber'
        db.delete_table(db.shorten_name(u'unisender_subscriber_list_ids'))

        # Removing M2M table for field tags on 'Subscriber'
        db.delete_table(db.shorten_name(u'unisender_subscriber_tags'))

        # Deleting model 'SubscriberFields'
        db.delete_table(u'unisender_subscriberfields')

        # Deleting model 'EmailMessage'
        db.delete_table(u'unisender_emailmessage')

        # Deleting model 'SmsMessage'
        db.delete_table(u'unisender_smsmessage')

        # Deleting model 'Campaign'
        db.delete_table(u'unisender_campaign')

        # Removing M2M table for field contacts on 'Campaign'
        db.delete_table(db.shorten_name(u'unisender_campaign_contacts'))


    models = {
        u'unisender.campaign': {
            'Meta': {'ordering': "('name',)", 'object_name': 'Campaign'},
            'contacts': ('django.db.models.fields.related.ManyToManyField', [], {'blank': 'True', 'related_name': "'campaign'", 'null': 'True', 'symmetrical': 'False', 'to': u"orm['unisender.Subscriber']"}),
            'email_message': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['unisender.EmailMessage']"}),
            'err_delivery_failed': ('django.db.models.fields.PositiveSmallIntegerField', [], {'default': '0'}),
            'err_dest_invalid': ('django.db.models.fields.PositiveSmallIntegerField', [], {'default': '0'}),
            'err_domain_inactive': ('django.db.models.fields.PositiveSmallIntegerField', [], {'default': '0'}),
            'err_internal': ('django.db.models.fields.PositiveSmallIntegerField', [], {'default': '0'}),
            'err_lost': ('django.db.models.fields.PositiveSmallIntegerField', [], {'default': '0'}),
            'err_mailbox_full': ('django.db.models.fields.PositiveSmallIntegerField', [], {'default': '0'}),
            'err_not_allowed': ('django.db.models.fields.PositiveSmallIntegerField', [], {'default': '0'}),
            'err_not_available': ('django.db.models.fields.PositiveSmallIntegerField', [], {'default': '0'}),
            'err_resend': ('django.db.models.fields.PositiveSmallIntegerField', [], {'default': '0'}),
            'err_skip_letter': ('django.db.models.fields.PositiveSmallIntegerField', [], {'default': '0'}),
            'err_spam_folder': ('django.db.models.fields.PositiveSmallIntegerField', [], {'default': '0'}),
            'err_spam_rejected': ('django.db.models.fields.PositiveSmallIntegerField', [], {'default': '0'}),
            'err_spam_retry': ('django.db.models.fields.PositiveSmallIntegerField', [], {'default': '0'}),
            'err_spam_skipped': ('django.db.models.fields.PositiveSmallIntegerField', [], {'default': '0'}),
            'err_src_invalid': ('django.db.models.fields.PositiveSmallIntegerField', [], {'default': '0'}),
            'err_unsubscribed': ('django.db.models.fields.PositiveSmallIntegerField', [], {'default': '0'}),
            'err_user_inactive': ('django.db.models.fields.PositiveSmallIntegerField', [], {'default': '0'}),
            'err_user_unknown': ('django.db.models.fields.PositiveSmallIntegerField', [], {'default': '0'}),
            'err_will_retry': ('django.db.models.fields.PositiveSmallIntegerField', [], {'default': '0'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'last_check': ('django.db.models.fields.DateTimeField', [], {'null': 'True', 'blank': 'True'}),
            'last_error': ('django.db.models.fields.CharField', [], {'max_length': '255', 'null': 'True', 'blank': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '255'}),
            'not_sent': ('django.db.models.fields.PositiveSmallIntegerField', [], {'default': '0'}),
            'ok_delivered': ('django.db.models.fields.PositiveSmallIntegerField', [], {'default': '0'}),
            'ok_link_visited': ('django.db.models.fields.PositiveSmallIntegerField', [], {'default': '0'}),
            'ok_read': ('django.db.models.fields.PositiveSmallIntegerField', [], {'default': '0'}),
            'ok_spam_folder': ('django.db.models.fields.PositiveSmallIntegerField', [], {'default': '0'}),
            'ok_unsubscribed': ('django.db.models.fields.PositiveSmallIntegerField', [], {'default': '0'}),
            'payment_limit': ('django.db.models.fields.PositiveSmallIntegerField', [], {'null': 'True', 'blank': 'True'}),
            'start_time': ('django.db.models.fields.DateTimeField', [], {'null': 'True', 'blank': 'True'}),
            'status': ('django.db.models.fields.CharField', [], {'default': 'None', 'max_length': '50', 'null': 'True', 'blank': 'True'}),
            'sync': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'total': ('django.db.models.fields.PositiveSmallIntegerField', [], {'default': '0'}),
            'track_ga': ('django.db.models.fields.CharField', [], {'default': "'0'", 'max_length': '50'}),
            'track_links': ('django.db.models.fields.CharField', [], {'default': "'0'", 'max_length': '50'}),
            'track_read': ('django.db.models.fields.CharField', [], {'default': "'0'", 'max_length': '50'}),
            'unisender_id': ('django.db.models.fields.CharField', [], {'max_length': '255', 'null': 'True', 'blank': 'True'})
        },
        u'unisender.emailmessage': {
            'Meta': {'ordering': "('subject',)", 'object_name': 'EmailMessage'},
            'body': ('tinymce_4.fields.TinyMCEModelField', [], {}),
            'categories': ('django.db.models.fields.CharField', [], {'max_length': '255', 'null': 'True', 'blank': 'True'}),
            'generate_text': ('django.db.models.fields.CharField', [], {'default': "'1'", 'max_length': '50'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'lang': ('django.db.models.fields.CharField', [], {'default': "'ru'", 'max_length': '50'}),
            'last_error': ('django.db.models.fields.CharField', [], {'max_length': '255', 'null': 'True', 'blank': 'True'}),
            'list_id': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'emails'", 'to': u"orm['unisender.SubscribeList']"}),
            'sender_email': ('django.db.models.fields.CharField', [], {'max_length': '255'}),
            'sender_name': ('django.db.models.fields.CharField', [], {'max_length': '255'}),
            'series_day': ('django.db.models.fields.PositiveSmallIntegerField', [], {'null': 'True', 'blank': 'True'}),
            'series_time': ('django.db.models.fields.TimeField', [], {'default': 'datetime.datetime(2014, 7, 8, 0, 0)'}),
            'subject': ('django.db.models.fields.CharField', [], {'max_length': '255'}),
            'sync': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'tag': ('django.db.models.fields.related.ForeignKey', [], {'blank': 'True', 'related_name': "'emails'", 'null': 'True', 'to': u"orm['unisender.Tag']"}),
            'text_body': ('django.db.models.fields.TextField', [], {'null': 'True', 'blank': 'True'}),
            'unisender_id': ('django.db.models.fields.CharField', [], {'max_length': '255', 'null': 'True', 'blank': 'True'}),
            'wrap_type': ('django.db.models.fields.CharField', [], {'default': "'skip'", 'max_length': '50'})
        },
        u'unisender.field': {
            'Meta': {'ordering': "('name',)", 'object_name': 'Field'},
            'field_type': ('django.db.models.fields.CharField', [], {'default': "'string'", 'max_length': '50'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'last_error': ('django.db.models.fields.CharField', [], {'max_length': '255', 'null': 'True', 'blank': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '255'}),
            'sort': ('django.db.models.fields.SmallIntegerField', [], {'default': '1'}),
            'sync': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'unisender_id': ('django.db.models.fields.CharField', [], {'max_length': '255', 'null': 'True', 'blank': 'True'}),
            'visible': ('django.db.models.fields.BooleanField', [], {'default': 'True'})
        },
        u'unisender.smsmessage': {
            'Meta': {'object_name': 'SmsMessage'},
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'last_error': ('django.db.models.fields.CharField', [], {'max_length': '255', 'null': 'True', 'blank': 'True'}),
            'sync': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'unisender_id': ('django.db.models.fields.CharField', [], {'max_length': '255', 'null': 'True', 'blank': 'True'})
        },
        u'unisender.subscribelist': {
            'Meta': {'ordering': "('title',)", 'object_name': 'SubscribeList'},
            'after_subscribe_url': ('django.db.models.fields.CharField', [], {'max_length': '255', 'null': 'True', 'blank': 'True'}),
            'before_subscribe_url': ('django.db.models.fields.CharField', [], {'max_length': '255', 'null': 'True', 'blank': 'True'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'last_error': ('django.db.models.fields.CharField', [], {'max_length': '255', 'null': 'True', 'blank': 'True'}),
            'sync': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'title': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '255'}),
            'unisender_id': ('django.db.models.fields.CharField', [], {'max_length': '255', 'null': 'True', 'blank': 'True'})
        },
        u'unisender.subscriber': {
            'Meta': {'ordering': "('contact',)", 'object_name': 'Subscriber'},
            'contact': ('django.db.models.fields.CharField', [], {'max_length': '255'}),
            'contact_type': ('django.db.models.fields.CharField', [], {'default': "'email'", 'max_length': '50'}),
            'double_optin': ('django.db.models.fields.SmallIntegerField', [], {'default': '1'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'last_error': ('django.db.models.fields.CharField', [], {'max_length': '255', 'null': 'True', 'blank': 'True'}),
            'list_ids': ('django.db.models.fields.related.ManyToManyField', [], {'related_name': "'subscribers'", 'symmetrical': 'False', 'to': u"orm['unisender.SubscribeList']"}),
            'sync': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'tags': ('django.db.models.fields.related.ManyToManyField', [], {'blank': 'True', 'related_name': "'subscribers'", 'null': 'True', 'symmetrical': 'False', 'to': u"orm['unisender.Tag']"}),
            'unisender_id': ('django.db.models.fields.CharField', [], {'max_length': '255', 'null': 'True', 'blank': 'True'})
        },
        u'unisender.subscriberfields': {
            'Meta': {'object_name': 'SubscriberFields'},
            'field': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['unisender.Field']"}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'subscriber': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'fields'", 'to': u"orm['unisender.Subscriber']"}),
            'value': ('django.db.models.fields.CharField', [], {'max_length': '255'})
        },
        u'unisender.tag': {
            'Meta': {'ordering': "('name',)", 'object_name': 'Tag'},
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'last_error': ('django.db.models.fields.CharField', [], {'max_length': '255', 'null': 'True', 'blank': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '255'}),
            'sync': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'unisender_id': ('django.db.models.fields.CharField', [], {'max_length': '255', 'null': 'True', 'blank': 'True'})
        }
    }

    complete_apps = ['unisender']


# -*- coding: utf-8 -*-
from django.contrib import admin
from django.conf.urls import patterns, url
from django.contrib import messages
from django.utils.translation import ugettext_lazy as _
from django import forms
from django.core.exceptions import ValidationError

from unisender.models import (
    Tag, Field, SubscribeList, Subscriber, SubscriberFields,
    EmailMessage, Campaign, Attachment, VisitedLink, OptinEmail)

from unisender.unisender_urls import (
    EMAIL_MESSAGES_LIST, EMAIL_MESSAGES_DETAIL, TAG_LIST, FIELD_LIST,
    CAMPAIGN_LIST, CAMPAIGN_DETAIL, SUBSCRIBELIST_LIST, SUBSCRIBELIST_DETAIL
)

from unisender.views import (
    GetCampaignStatistic, GetTags, GetFields, GetLists, GetCampaigns
)

unisender_fieldsets = [
    [u'Unisender', {
     'fields': ['unisender_id', 'sync', 'get_last_error']
     }]
]


class UnisenderAdmin(admin.ModelAdmin):
    readonly_fields = ['unisender_id', 'sync', 'get_last_error']

    fieldsets = unisender_fieldsets


class TagAdmin(UnisenderAdmin):
    fieldsets = unisender_fieldsets + [[u'Метка', {'fields': ['name', ]}]]

    list_display = ('__unicode__', 'unisender_id', 'sync', )
    list_display_links = ('__unicode__', )
    search_fields = ['name', ]
    change_list_template = 'unisender/admin/change_tag_list.html'

    def get_urls(self):
        urls = super(TagAdmin, self).get_urls()
        my_urls = patterns('',
                           url(r'get_tags/$',
                               self.admin_site.admin_view(GetTags.as_view()),
                               name='unisender_get_tags',
                               ),
                           )
        return my_urls + urls

    def changelist_view(self, request, extra_context=None):
        extra_context = extra_context or {}
        extra_context['unisender_site'] = TAG_LIST
        return super(TagAdmin, self).changelist_view(request,
                                                     extra_context=extra_context)

admin.site.register(Tag, TagAdmin)


class FieldAdmin(UnisenderAdmin):
    fieldsets = unisender_fieldsets + [[u'Поле', {
        'fields': ['name', 'field_type', 'visible', 'sort']
    }]]

    list_display = ('__unicode__', 'unisender_id', 'sync', 'field_type',
                    'visible', 'sort')
    list_display_links = ('__unicode__', )
    search_fields = ['name', ]
    list_editable = ('field_type', 'visible', 'sort')

    actions = ['delete_selected_fields']

    change_list_template = 'unisender/admin/change_field_list.html'

    def get_urls(self):
        urls = super(FieldAdmin, self).get_urls()
        my_urls = patterns('',
                           url(r'get_fields/$',
                               self.admin_site.admin_view(GetFields.as_view()),
                               name='unisender_get_fields',
                               ),
                           )
        return my_urls + urls

    def get_actions(self, request):
        actions = super(FieldAdmin, self).get_actions(request)
        del actions['delete_selected']
        return actions

    def delete_selected_fields(self, request, queryset):
        for item in queryset:
            self.delete_model(request, item)
    delete_selected_fields.short_description = u'Удалить выбранные Поля'

    def save_model(self, request, obj, form, change):
        if obj.pk:
            if obj.unisender_id:
                obj.update_field(request)
            else:
                obj.unisender_id = obj.create_field(request)
        else:
            obj.unisender_id = obj.create_field(request)
        obj.save()

    def delete_model(self, request, obj):
        obj.delete_field(request)
        obj.delete()

    def changelist_view(self, request, extra_context=None):
        extra_context = extra_context or {}
        extra_context['unisender_site'] = FIELD_LIST
        return super(FieldAdmin, self).changelist_view(request,
                                                       extra_context=extra_context)

admin.site.register(Field, FieldAdmin)


class OptinEmailInline(admin.StackedInline):
    model = OptinEmail
    can_delete = False
    readonly_fields = ['unisender_id', 'sync', 'get_last_error']
    fieldsets = [[u'Unisender', {
                 'fields': ['sync', 'get_last_error']
                 }],
                 (u'Письмо', {
                  'fields': ['sender_name', 'sender_email', 'subject', 'body',]
                  })]

    # def save_model(self, request, obj, form, change):

class SubscribeListAdmin(UnisenderAdmin):
    fieldsets = unisender_fieldsets + [
        (u'Список рассылки', {
            'fields': ['title', 'before_subscribe_url', 'after_subscribe_url']
        })]

    list_display = ('__unicode__', 'unisender_id', 'sync',
                    'before_subscribe_url', 'after_subscribe_url')
    list_display_links = ('__unicode__', )
    search_fields = ['title', ]
    change_list_template = 'unisender/admin/change_subscriber_list_list.html'
    change_form_template = 'unisender/admin/change_subscriber_list_detail.html'
    inlines = [OptinEmailInline]

    def get_urls(self):
        urls = super(SubscribeListAdmin, self).get_urls()
        my_urls = patterns('',
                           url(r'get_lists/$',
                               self.admin_site.admin_view(GetLists.as_view()),
                               name='unisender_get_lists',
                               ),
                           )
        return my_urls + urls

    def save_model(self, request, obj, form, change):
        if obj.unisender_id:
            if obj.pk:
                obj.update_list(request)
        else:
            obj.unisender_id = obj.create_list(request)
        obj.save()

    actions = ['delete_selected_subscribe_list']

    def get_actions(self, request):
        actions = super(SubscribeListAdmin, self).get_actions(request)
        del actions['delete_selected']
        return actions

    def delete_selected_subscribe_list(self, request, queryset):
        for item in queryset:
            self.delete_model(request, item)
    delete_selected_subscribe_list.short_description = u'Удалить выбранные Списки подписчиков'

    def delete_model(self, request, obj):
        obj.delete_list(request)
        obj.delete()

    def changelist_view(self, request, extra_context=None):
        extra_context = extra_context or {}
        extra_context['unisender_site'] = SUBSCRIBELIST_LIST
        return super(SubscribeListAdmin, self).changelist_view(request,
                                                               extra_context=extra_context)

    def change_view(self, request, object_id, form_url='', extra_context=None):
        extra_context = extra_context or {}
        instance = self.model.objects.get(pk=object_id)
        if instance.unisender_id:
            extra_context['unisender_site'] =\
                SUBSCRIBELIST_DETAIL + instance.unisender_id
        self.inlines = [OptinEmailInline, ]
        return super(SubscribeListAdmin, self).change_view(
            request, object_id, form_url, extra_context)

    def add_view(self, request, form_url='', extra_context=None):
        self.inlines = []
        return super(SubscribeListAdmin, self).add_view(
            request, form_url, extra_context)

    def save_related(self, request, form, formsets, change):
        super(type(self), self).save_related(request, form, formsets, change)
        if change and formsets[0].queryset:
            instance = formsets[0].queryset[0]
            instance.update_optin_email(request)
            instance.save()

admin.site.register(SubscribeList, SubscribeListAdmin)


class SubscriberFieldsInline(admin.TabularInline):
    model = SubscriberFields
    extra = 0


class SubscriberAdmin(UnisenderAdmin):
    fieldsets = unisender_fieldsets + [
        (u'Подписчик', {
            'fields': ['list_ids', 'tags', 'contact_type', 'contact',
                       'double_optin']
        })]

    list_display = ('__unicode__', 'contact_type', 'unisender_id', 'sync')
    list_display_links = ('__unicode__', )
    search_fields = ['contact', ]
    filter_horizontal = ['list_ids', 'tags']
    inlines = [SubscriberFieldsInline]

    def get_readonly_fields(self, request, obj=None):
        readonly_fields = super(SubscriberAdmin, self).get_readonly_fields(
            request, obj=obj)
        if obj and obj.sync:
            readonly_fields += ['contact', 'contact_type']
        else:
            # save and add another button behavior
            readonly_fields = list(set(readonly_fields))
            if 'contact' in readonly_fields:
                readonly_fields.remove('contact')
            if 'contact_type' in readonly_fields:
                readonly_fields.remove('contact_type')

        return readonly_fields

    def save_model(self, request, obj, form, change):
        obj.save()
        if not obj.unisender_id:
            obj.unisender_id = obj.subscribe(request)
        obj.save()

    actions = ['delete_selected_subscribers']

    def get_actions(self, request):
        actions = super(SubscriberAdmin, self).get_actions(request)
        del actions['delete_selected']
        return actions

    def delete_selected_subscribers(self, request, queryset):
        for item in queryset:
            self.delete_model(request, item)
    delete_selected_subscribers.short_description = u'Удалить выбранных Подписчиков'

    def delete_model(self, request, obj):
        obj.exclude(request)
        obj.delete()

admin.site.register(Subscriber, SubscriberAdmin)

auto_send_fieldset = [[u'Автоматическая отправка', {
    'fields': ['series_day', 'series_time', ]
}]]


class EmailMessageForm(forms.ModelForm):

    def __init__(self, *args, **kwargs):
        super(EmailMessageForm, self).__init__(*args, **kwargs)
        if not self.instance.unisender_id:
            self.fields['list_id'].required = True


class AttachmentInline(admin.TabularInline):
    model = Attachment
    extra = 0


class AttachmentInlineReadOnly(AttachmentInline):
    can_delete = False

    def has_add_permission(self, request):
        return False

    def get_readonly_fields(self, request, obj=None):
        result = list(set(
            [field.name for field in self.opts.local_fields] +
            [field.name for field in self.opts.local_many_to_many]
        ))
        result.remove('id')
        return result


class EmailMessageAdmin(UnisenderAdmin):
    fieldsets = unisender_fieldsets + [
        [u'Сообщение', {
            'fields': ['sender_name', 'sender_email', 'subject', 'body', 'list_id',
                       'lang', 'text_body', 'generate_text', 'wrap_type',
                       'categories', 'tag']
        }]] + auto_send_fieldset

    list_display = (
        '__unicode__', 'sender_name', 'subject', 'unisender_id', 'sync')
    list_display_links = ('__unicode__', )
    search_fields = ['sender_name', 'subject', 'body', ]
    change_list_template = 'unisender/admin/change_emailmessage_list.html'
    change_form_template = 'unisender/admin/change_emailmessage.html'
    form = EmailMessageForm

    def get_readonly_fields(self, request, obj=None):
        if obj and obj.sync:
            result = list(set(
                [field.name for field in self.opts.local_fields] +
                [field.name for field in self.opts.local_many_to_many]
            ))
            result += ['get_last_error']
            result += ['read_only_body']
            return result
        return super(EmailMessageAdmin, self).get_readonly_fields(request, obj=None)

    def get_fieldsets(self, request, obj=None):
        field_sets = super(
            EmailMessageAdmin, self).get_fieldsets(request, obj=None)
        if obj and obj.unisender_id:
            field_sets = unisender_fieldsets + [
                [u'Сообщение', {
                 'fields': ['sender_name', 'sender_email', 'subject',
                            'read_only_body', 'list_id', 'lang', 'text_body',
                            'generate_text', 'wrap_type', 'categories', 'tag']
                 }]] + auto_send_fieldset
        return field_sets

    def response_add(self, request, obj):
        if not obj.unisender_id:
            obj.unisender_id = obj.create_email_message(request)
        obj.save()
        return super(EmailMessageAdmin, self).response_add(request, obj)

    def save_formset(self, request, form, formset, change):
        instances = formset.save(commit=False)
        for instance in instances:
            instance.user = request.user
            instance.save()
        formset.save_m2m()

    actions = ['delete_selected_emails']

    def get_actions(self, request):
        actions = super(EmailMessageAdmin, self).get_actions(request)
        del actions['delete_selected']
        return actions

    def delete_selected_emails(self, request, queryset):
        for item in queryset:
            self.delete_model(request, item)
    delete_selected_emails.short_description = u'Удалить выбранные сообщения электронной почты'

    def delete_model(self, request, obj):
        obj.delete_message(request)
        obj.delete()

    def add_view(self, request, form_url='', extra_context=None):
        self.inlines = [AttachmentInline, ]
        return super(EmailMessageAdmin, self).add_view(
            request, form_url, extra_context)

    def change_view(self, request, object_id, form_url='', extra_context=None):
        self.inlines = [AttachmentInlineReadOnly, ]
        extra_context = extra_context or {}
        instance = self.model.objects.get(pk=object_id)
        if instance.unisender_id:
            extra_context['unisender_site'] = \
                EMAIL_MESSAGES_DETAIL + instance.unisender_id
        return super(EmailMessageAdmin, self).change_view(
            request, object_id, form_url, extra_context)

    def changelist_view(self, request, extra_context=None):
        extra_context = extra_context or {}
        extra_context['unisender_site'] = EMAIL_MESSAGES_LIST
        return super(EmailMessageAdmin, self).changelist_view(request,
                                                              extra_context=extra_context)

admin.site.register(EmailMessage, EmailMessageAdmin)


# admin.site.register(SmsMessage)


class CampaignAdminForm(forms.ModelForm):

    def clean_contacts(self):
        email_message = self.cleaned_data.get('email_message', None)
        if not email_message:
            return self.cleaned_data['contacts']
        if self.cleaned_data['contacts'] or (email_message and email_message.list_id):
            return self.cleaned_data['contacts']
        else:
            raise ValidationError(
                u'''У выбранного сообщения отсутствует список контактов,
                    вам необходимо выбрать контакты которым будет осуществлена рассылка''')

    def __init__(self, *args, **kwargs):
        super(CampaignAdminForm, self).__init__(*args, **kwargs)
        if not self.instance.unisender_id:
            self.fields['name'].required = True
            self.fields['email_message'].required = True


class CampaignVisitedLinksInline(admin.TabularInline):
    model = VisitedLink
    extra = 0
    can_delete = False
    fields = ('email', 'ip', 'url', 'request_time', 'count')

    def has_add_permission(self, request):
        return False

    def get_readonly_fields(self, request, obj=None):
        result = list(set(
            [field.name for field in self.opts.local_fields] +
            [field.name for field in self.opts.local_many_to_many]
        ))
        result.remove('id')
        return result


class CampaignAdmin(UnisenderAdmin):
    change_form_template = 'unisender/admin/change_campaign.html'
    fieldsets = unisender_fieldsets + [
        (u'Рассылка', {
            'fields': ['name', 'email_message', 'start_time',
                       'track_read', 'track_links', 'contacts', 'track_ga',
                       'payment_limit', ]
        })]

    list_display = (
        '__unicode__', 'email_message', 'unisender_id', 'sync')
    list_display_links = ('__unicode__',  'email_message', 'unisender_id')
    search_fields = ['name', 'contacts', ]
    filter_horizontal = ['contacts']
    change_list_template = 'unisender/admin/change_campaign_list.html'
    inlines = [CampaignVisitedLinksInline, ]

    form = CampaignAdminForm

    def get_readonly_fields(self, request, obj=None):
        if obj and obj.sync:
            result = list(set(
                [field.name for field in self.opts.local_fields] +
                [field.name for field in self.opts.local_many_to_many]
            ))
            result += [
                'get_last_error', 'get_error_count', 'get_success_count']
            return result
        return super(CampaignAdmin, self).get_readonly_fields(request, obj=None)

    def get_fieldsets(self, request, obj=None):
        field_sets = super(
            CampaignAdmin, self).get_fieldsets(request, obj=None)
        if obj and obj.sync:
            field_sets = field_sets + [
                (u'Статус', {
                 'fields': ['status', 'start_time', ]
                 }),
                (u'Краткая информация', {
                 'fields': ['get_error_count', 'get_success_count']
                 }),
                (u'Подробная информация', {
                 'fields': ['not_sent', 'ok_delivered', 'ok_read',
                            'ok_spam_folder', 'ok_link_visited',
                            'ok_unsubscribed',
                            'err_user_unknown', 'err_user_inactive',
                            'err_mailbox_full', 'err_spam_rejected',
                            'err_spam_folder', 'err_delivery_failed',
                            'err_will_retry', 'err_resend', 'err_not_allowed',
                            'err_domain_inactive', 'err_unsubscribed',
                            'err_skip_letter', 'err_spam_retry',
                            'err_src_invalid', 'err_dest_invalid',
                            'err_not_available', 'err_lost', 'err_internal', ]
                 })]
        return field_sets

    def change_view(self, request, object_id, form_url='', extra_context=None):
        extra_context = extra_context or {}
        campaign = Campaign.objects.get(pk=object_id)
        instance = self.model.objects.get(pk=object_id)
        if instance.unisender_id:
            extra_context['unisender_site'] =\
                CAMPAIGN_DETAIL + instance.unisender_id
        if campaign.sync:
            extra_context['show_get_statistic_button'] = True
            extra_context['pk'] = object_id
        return super(CampaignAdmin, self).change_view(request, object_id,
                                                      form_url, extra_context=extra_context)

    def get_urls(self):
        urls = super(CampaignAdmin, self).get_urls()
        my_urls = patterns('',
                           url(r'(?P<pk>\d+)/get_statistic/$',
                               self.admin_site.admin_view(
                               GetCampaignStatistic.as_view()),
                               name='unisender_campaign_get_statistic',
                               ),
                           url(r'get_campaigns/$',
                               self.admin_site.admin_view(
                                   GetCampaigns.as_view()),
                               name='unisender_get_campaigns',
                               ),

                           )
        return my_urls + urls

    def save_model(self, request, obj, form, change):
        obj.save()
        if not obj.unisender_id:
            obj.unisender_id = obj.create_campaign(request)
        obj.save()

    actions = ['delete_selected_campaigns']

    def get_actions(self, request):
        actions = super(CampaignAdmin, self).get_actions(request)
        del actions['delete_selected']
        return actions

    def delete_selected_campaigns(self, request, queryset):
        for item in queryset:
            self.delete_model(request, item)
    delete_selected_campaigns.short_description = u'Удалить выбранные Поля'

    def delete_model(self, request, obj):
        messages.warning(
            request, _(u'Объект был удален из БД сайта, но остался в БД unisender вам необходимо удалить его самостоятельно оттуда'))
        obj.delete()

    def changelist_view(self, request, extra_context=None):
        extra_context = extra_context or {}
        extra_context['unisender_site'] = CAMPAIGN_LIST
        return super(CampaignAdmin, self).changelist_view(request,
                                                          extra_context=extra_context)

admin.site.register(Campaign, CampaignAdmin)



# -*- coding: utf-8 -*-
from south.utils import datetime_utils as datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models


class Migration(SchemaMigration):

    def forwards(self, orm):

        # Changing field 'Subscriber.double_optin'
        db.alter_column(u'unisender_subscriber', 'double_optin', self.gf('django.db.models.fields.CharField')(max_length=2))

    def backwards(self, orm):

        # Changing field 'Subscriber.double_optin'
        db.alter_column(u'unisender_subscriber', 'double_optin', self.gf('django.db.models.fields.SmallIntegerField')())

    models = {
        u'unisender.campaign': {
            'Meta': {'ordering': "('name',)", 'object_name': 'Campaign'},
            'contacts': ('django.db.models.fields.related.ManyToManyField', [], {'blank': 'True', 'related_name': "'campaign'", 'null': 'True', 'symmetrical': 'False', 'to': u"orm['unisender.Subscriber']"}),
            'email_message': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['unisender.EmailMessage']"}),
            'err_delivery_failed': ('django.db.models.fields.PositiveSmallIntegerField', [], {'default': '0'}),
            'err_dest_invalid': ('django.db.models.fields.PositiveSmallIntegerField', [], {'default': '0'}),
            'err_domain_inactive': ('django.db.models.fields.PositiveSmallIntegerField', [], {'default': '0'}),
            'err_internal': ('django.db.models.fields.PositiveSmallIntegerField', [], {'default': '0'}),
            'err_lost': ('django.db.models.fields.PositiveSmallIntegerField', [], {'default': '0'}),
            'err_mailbox_full': ('django.db.models.fields.PositiveSmallIntegerField', [], {'default': '0'}),
            'err_not_allowed': ('django.db.models.fields.PositiveSmallIntegerField', [], {'default': '0'}),
            'err_not_available': ('django.db.models.fields.PositiveSmallIntegerField', [], {'default': '0'}),
            'err_resend': ('django.db.models.fields.PositiveSmallIntegerField', [], {'default': '0'}),
            'err_skip_letter': ('django.db.models.fields.PositiveSmallIntegerField', [], {'default': '0'}),
            'err_spam_folder': ('django.db.models.fields.PositiveSmallIntegerField', [], {'default': '0'}),
            'err_spam_rejected': ('django.db.models.fields.PositiveSmallIntegerField', [], {'default': '0'}),
            'err_spam_retry': ('django.db.models.fields.PositiveSmallIntegerField', [], {'default': '0'}),
            'err_spam_skipped': ('django.db.models.fields.PositiveSmallIntegerField', [], {'default': '0'}),
            'err_src_invalid': ('django.db.models.fields.PositiveSmallIntegerField', [], {'default': '0'}),
            'err_unsubscribed': ('django.db.models.fields.PositiveSmallIntegerField', [], {'default': '0'}),
            'err_user_inactive': ('django.db.models.fields.PositiveSmallIntegerField', [], {'default': '0'}),
            'err_user_unknown': ('django.db.models.fields.PositiveSmallIntegerField', [], {'default': '0'}),
            'err_will_retry': ('django.db.models.fields.PositiveSmallIntegerField', [], {'default': '0'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'last_check': ('django.db.models.fields.DateTimeField', [], {'null': 'True', 'blank': 'True'}),
            'last_error': ('django.db.models.fields.CharField', [], {'max_length': '255', 'null': 'True', 'blank': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '255'}),
            'not_sent': ('django.db.models.fields.PositiveSmallIntegerField', [], {'default': '0'}),
            'ok_delivered': ('django.db.models.fields.PositiveSmallIntegerField', [], {'default': '0'}),
            'ok_link_visited': ('django.db.models.fields.PositiveSmallIntegerField', [], {'default': '0'}),
            'ok_read': ('django.db.models.fields.PositiveSmallIntegerField', [], {'default': '0'}),
            'ok_spam_folder': ('django.db.models.fields.PositiveSmallIntegerField', [], {'default': '0'}),
            'ok_unsubscribed': ('django.db.models.fields.PositiveSmallIntegerField', [], {'default': '0'}),
            'payment_limit': ('django.db.models.fields.PositiveSmallIntegerField', [], {'null': 'True', 'blank': 'True'}),
            'start_time': ('django.db.models.fields.DateTimeField', [], {'null': 'True', 'blank': 'True'}),
            'status': ('django.db.models.fields.CharField', [], {'default': 'None', 'max_length': '50', 'null': 'True', 'blank': 'True'}),
            'sync': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'total': ('django.db.models.fields.PositiveSmallIntegerField', [], {'default': '0'}),
            'track_ga': ('django.db.models.fields.CharField', [], {'default': "'0'", 'max_length': '50'}),
            'track_links': ('django.db.models.fields.CharField', [], {'default': "'0'", 'max_length': '50'}),
            'track_read': ('django.db.models.fields.CharField', [], {'default': "'0'", 'max_length': '50'}),
            'unisender_id': ('django.db.models.fields.CharField', [], {'max_length': '255', 'null': 'True', 'blank': 'True'})
        },
        u'unisender.emailmessage': {
            'Meta': {'ordering': "('subject',)", 'object_name': 'EmailMessage'},
            'body': ('tinymce_4.fields.TinyMCEModelField', [], {}),
            'categories': ('django.db.models.fields.CharField', [], {'max_length': '255', 'null': 'True', 'blank': 'True'}),
            'generate_text': ('django.db.models.fields.CharField', [], {'default': "'1'", 'max_length': '50'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'lang': ('django.db.models.fields.CharField', [], {'default': "'ru'", 'max_length': '50'}),
            'last_error': ('django.db.models.fields.CharField', [], {'max_length': '255', 'null': 'True', 'blank': 'True'}),
            'list_id': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'emails'", 'to': u"orm['unisender.SubscribeList']"}),
            'sender_email': ('django.db.models.fields.CharField', [], {'max_length': '255'}),
            'sender_name': ('django.db.models.fields.CharField', [], {'max_length': '255'}),
            'series_day': ('django.db.models.fields.PositiveSmallIntegerField', [], {'null': 'True', 'blank': 'True'}),
            'series_time': ('django.db.models.fields.TimeField', [], {'default': 'datetime.datetime(2014, 7, 21, 0, 0)'}),
            'subject': ('django.db.models.fields.CharField', [], {'max_length': '255'}),
            'sync': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'tag': ('django.db.models.fields.related.ForeignKey', [], {'blank': 'True', 'related_name': "'emails'", 'null': 'True', 'to': u"orm['unisender.Tag']"}),
            'text_body': ('django.db.models.fields.TextField', [], {'null': 'True', 'blank': 'True'}),
            'unisender_id': ('django.db.models.fields.CharField', [], {'max_length': '255', 'null': 'True', 'blank': 'True'}),
            'wrap_type': ('django.db.models.fields.CharField', [], {'default': "'skip'", 'max_length': '50'})
        },
        u'unisender.field': {
            'Meta': {'ordering': "('name',)", 'object_name': 'Field'},
            'field_type': ('django.db.models.fields.CharField', [], {'default': "'string'", 'max_length': '50'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'last_error': ('django.db.models.fields.CharField', [], {'max_length': '255', 'null': 'True', 'blank': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '255'}),
            'sort': ('django.db.models.fields.SmallIntegerField', [], {'default': '1'}),
            'sync': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'unisender_id': ('django.db.models.fields.CharField', [], {'max_length': '255', 'null': 'True', 'blank': 'True'}),
            'visible': ('django.db.models.fields.BooleanField', [], {'default': 'True'})
        },
        u'unisender.smsmessage': {
            'Meta': {'object_name': 'SmsMessage'},
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'last_error': ('django.db.models.fields.CharField', [], {'max_length': '255', 'null': 'True', 'blank': 'True'}),
            'sync': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'unisender_id': ('django.db.models.fields.CharField', [], {'max_length': '255', 'null': 'True', 'blank': 'True'})
        },
        u'unisender.subscribelist': {
            'Meta': {'ordering': "('title',)", 'object_name': 'SubscribeList'},
            'after_subscribe_url': ('django.db.models.fields.CharField', [], {'max_length': '255', 'null': 'True', 'blank': 'True'}),
            'before_subscribe_url': ('django.db.models.fields.CharField', [], {'max_length': '255', 'null': 'True', 'blank': 'True'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'last_error': ('django.db.models.fields.CharField', [], {'max_length': '255', 'null': 'True', 'blank': 'True'}),
            'sync': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'title': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '255'}),
            'unisender_id': ('django.db.models.fields.CharField', [], {'max_length': '255', 'null': 'True', 'blank': 'True'})
        },
        u'unisender.subscriber': {
            'Meta': {'ordering': "('contact',)", 'object_name': 'Subscriber'},
            'contact': ('django.db.models.fields.CharField', [], {'max_length': '255'}),
            'contact_type': ('django.db.models.fields.CharField', [], {'default': "'email'", 'max_length': '50'}),
            'double_optin': ('django.db.models.fields.CharField', [], {'default': '1', 'max_length': '2'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'last_error': ('django.db.models.fields.CharField', [], {'max_length': '255', 'null': 'True', 'blank': 'True'}),
            'list_ids': ('django.db.models.fields.related.ManyToManyField', [], {'related_name': "'subscribers'", 'symmetrical': 'False', 'to': u"orm['unisender.SubscribeList']"}),
            'sync': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'tags': ('django.db.models.fields.related.ManyToManyField', [], {'blank': 'True', 'related_name': "'subscribers'", 'null': 'True', 'symmetrical': 'False', 'to': u"orm['unisender.Tag']"}),
            'unisender_id': ('django.db.models.fields.CharField', [], {'max_length': '255', 'null': 'True', 'blank': 'True'})
        },
        u'unisender.subscriberfields': {
            'Meta': {'object_name': 'SubscriberFields'},
            'field': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['unisender.Field']"}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'subscriber': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'fields'", 'to': u"orm['unisender.Subscriber']"}),
            'value': ('django.db.models.fields.CharField', [], {'max_length': '255'})
        },
        u'unisender.tag': {
            'Meta': {'ordering': "('name',)", 'object_name': 'Tag'},
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'last_error': ('django.db.models.fields.CharField', [], {'max_length': '255', 'null': 'True', 'blank': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '255'}),
            'sync': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'unisender_id': ('django.db.models.fields.CharField', [], {'max_length': '255', 'null': 'True', 'blank': 'True'})
        }
    }

    complete_apps = ['unisender']


# -*- coding: utf-8 -*-
import logging

from django.db import models
from django.contrib import messages
from django.utils.translation import ugettext_lazy as _

from pyunisend import PyUniSend
from settings import UNISENDER_API_KEY, UNISENDER_TEST_MODE

from error_codes import UNISENDER_COMMON_ERRORS

test_mode = 1 if UNISENDER_TEST_MODE else 0

logger = logging.getLogger(__name__)

class UnisenderManager(models.Manager):

    api = PyUniSend(UNISENDER_API_KEY, test_mode=test_mode)

    def log_warning(self, msg, request=None):
        if request:
            messages.warning(request, _(u'Сообщение при синхронизации с unisender: %s' % msg))
        logger.info(unicode(_(u'Сообщение при синхронизации с unisender: %s' % msg)))

    def success_message(self, message, request=None):
        if request:
            messages.success(request, message)

    def log_error(self, error, request=None):
        if request:
            messages.error(
                request,
                _(u'При синхронизации с unisender проиошла ошибка: %s' % error))
        logger.error(unicode(_(u'При синхронизации с unisender проиошла ошибка: %s' % error)))


class UnisenderTagManager(UnisenderManager):

    def get_tags(self):
        '''
        Возвращает результат выполнения команды getTags:
        http://www.unisender.com/ru/help/api/getTags/
        '''
        return self.api.getTags()

    def get_and_update_tags(self, request=None):
        '''
        Выполняет команду getTags и обновляет значения в БД:
        http://www.unisender.com/ru/help/api/getTags/
        '''
        tags = self.get_tags()
        result = tags.get('result', [])
        error = tags.get('error')
        warning = tags.get('warning')
        for item in result:
            tag, created = self.model.objects.get_or_create(name=item['name'])
            tag.unisender_id = int(item['id'])
            tag.sync = True
            tag.save()
        self.success_message(
            _(u'%s меток получено от unisender') % len(result),
            request=request)
        if error:
            error_msg = UNISENDER_COMMON_ERRORS.get(tags.get('code'), error)
            self.log_error(error_msg, request)
        if warning:
            self.log_warning(warning, request)


class UnisenderFieldManager(UnisenderManager):

    def get_fields(self):
        '''
        Возвращает результат выполнения команды getFields:
        http://www.unisender.com/ru/help/api/getFields/
        '''
        return self.api.getFields()

    def get_and_update_fields(self, request=None):
        '''
        Выполняет команду getFields и обновляет значения в БД:
        http://www.unisender.com/ru/help/api/getFields/
        '''
        fields = self.get_fields()
        result = fields.get('result', [])
        error = fields.get('error')
        warning = fields.get('warning')
        for item in result:
            field, create = self.model.objects.get_or_create(name=item['name'])
            field.unisender_id = item['id']
            field.sync = True
            field.visible = True if item['is_visible'] == 1 else False
            field.field_type = item['type']
            field.sort = item['view_pos']
            field.save()
        self.success_message(
            _(u'%s полей получено от unisender') % len(result),
            request=request)
        if error:
            error_msg = UNISENDER_COMMON_ERRORS.get(fields.get('code'), error)
            self.log_error(error_msg, request)
        if warning:
            self.log_warning(warning, request)

class UnisenderListManager(UnisenderManager):

    def get_lists(self):
        '''
        Возвращает результат выполнения команды getLists:
        http://www.unisender.com/ru/help/api/getLists/
        '''
        return self.api.getLists()

    def get_and_update_lists(self, request=None):
        '''
        Выполняет команду getLists и обновляет значения в БД:
        http://www.unisender.com/ru/help/api/getLists/
        '''
        lists = self.get_lists()
        result = lists.get('result', [])
        error = lists.get('error')
        warning = lists.get('warning')
        for item in result:
            tag, created = self.model.objects.get_or_create(title=item['title'])
            tag.unisender_id = int(item['id'])
            tag.sync = True
            tag.save()
        self.success_message(
            _(u'%s списков получено от unisender') % len(result),
            request=request)
        if error:
            error_msg = UNISENDER_COMMON_ERRORS.get(lists.get('code'), error)
            self.log_error(error_msg, request)
        if warning:
            self.log_warning(warning, request)


class UnisenderCampaignManager(UnisenderManager):

    def get_campaigns(self, from_arg=None, to_arg=None):
        '''
        Возвращает результат выполнения команды getCampaigns:
        http://www.unisender.com/ru/help/api/getCampaigns/
        '''
        params = {'from': from_arg, 'to': to_arg}
        return self.api.getCampaigns(params=params)

    def get_and_update_campaigns(self, request=None, from_arg=None, to_arg=None):
        '''
        Выполняет команду getCampaigns и обновляет значения в БД:
         http://www.unisender.com/ru/help/api/getCampaigns/
        '''
        campaigns = self.get_campaigns()
        result = campaigns.get('result', [])
        error = campaigns.get('error')
        warning = campaigns.get('warning')
        for item in result:
            tag, created = self.model.objects.get_or_create(
                unisender_id=int(item['id']))
            tag.sync = True
            tag.save()
        self.success_message(
            _(u'%s рассылок получено от unisender') % len(result),
            request=request)
        if error:
            error_msg = UNISENDER_COMMON_ERRORS.get(campaigns.get('code'), error)
            self.log_error(error_msg, request)
        if warning:
            self.log_warning(warning, request)
